@@ -1,13 +1,12 @@
 
 //OpenSCADA system file: main.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -60,11 +59,13 @@ int main(int argc, char *argv[], char *envp[] )
 	SYS = new TSYS(argc,argv,envp);	
 	
 	SYS->load();
-	rez = SYS->start();    
+	rez = SYS->start();
 	
 	delete SYS;
     }catch(TError err)
-    { Mess->put(err.cat.c_str(),TMess::Error,"%s",err.mess.c_str()); }
+    { mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
+
+    printf("OpenSCADA system is correct exited.\n");	
 
     return rez;
 }
