@@ -32,25 +32,23 @@
 
 #include <terror.h>
 #include <tsys.h>
-#include <resalloc.h>
 #include <tmess.h>
 #include <tconfig.h>
 #include <tvalue.h>
-#include <tparam.h>
-#include <tparams.h>
 #include <ttiparam.h>
 #include "virtual.h"
 
-//============ Modul info! =====================================================
+//************************************************
+//* Modul info!                                  *
 #define MOD_ID      "BlockCalc"
 #define MOD_NAME    "Block based calculator"
 #define MOD_TYPE    "DAQ"
 #define VER_TYPE    VER_CNTR
-#define VERSION     "0.9.0"
+#define VERSION     "1.0.0"
 #define AUTORS      "Roman Savochenko"
 #define DESCRIPTION "Allow block based calculator."
 #define LICENSE     "GPL"
-//==============================================================================
+//************************************************
 
 Virtual::TipContr *Virtual::mod;
 
@@ -58,37 +56,23 @@ extern "C"
 {
     TModule::SAt module( int n_mod )
     {
-	TModule::SAt AtMod;
-
-	if(n_mod==0)
-    	{
-	    AtMod.id	= MOD_ID;
-	    AtMod.type  = MOD_TYPE;
-	    AtMod.t_ver = VER_TYPE;
-	}
-	else
-	    AtMod.id	= "";
-
-	return( AtMod );
+	if( n_mod==0 )	return TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE);
+	return TModule::SAt("");
     }
 
     TModule *attach( const TModule::SAt &AtMod, const string &source )
     {
-	Virtual::TipContr *self_addr = NULL;
-
-	if( AtMod.id == MOD_ID && AtMod.type == MOD_TYPE && AtMod.t_ver == VER_TYPE )
-	    self_addr = Virtual::mod = new Virtual::TipContr( source );
-
-	return ( self_addr );
+	if( AtMod == TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE) )
+	    return new Virtual::TipContr( source );
+	return NULL;
     }
 }
 
 using namespace Virtual;
 
-//======================================================================
-//==== TipContr ======================================================== 
-//======================================================================
-
+//************************************************
+//* TipContr - BlockCalc type controller         *
+//************************************************ 
 TipContr::TipContr( string name ) 
 {
     mId 	= MOD_ID;
@@ -98,7 +82,9 @@ TipContr::TipContr( string name )
     mAutor    	= AUTORS;
     mDescr  	= DESCRIPTION;
     mLicense   	= LICENSE;
-    mSource    	= name;    
+    mSource    	= name;
+    
+    mod		= this;
 }
 
 TipContr::~TipContr()
@@ -110,7 +96,7 @@ string TipContr::optDescr( )
 {
     char buf[STR_BUF_LEN];
 
-    snprintf(buf,sizeof(buf),I18N(
+    snprintf(buf,sizeof(buf),_(
         "======================= The module <%s:%s> options =======================\n"
 	"---------- Parameters of the module section <%s> in config file ----------\n"),
 	MOD_TYPE,MOD_ID,nodePath().c_str());
@@ -142,37 +128,37 @@ void TipContr::modLoad()
     //========== Load parameters from config file =============
 }
 
-void TipContr::postEnable()
+void TipContr::postEnable( int flag )
 {    
-    TModule::postEnable();
+    TModule::postEnable( flag );
     
     //Controllers BD structure
-    fldAdd( new TFld("PRM_BD",I18N("Parameters table"),TFld::String,FLD_NOFLG,"30","system") );
-    fldAdd( new TFld("BLOCK_SH",I18N("Block's table"),TFld::String,FLD_NOFLG,"30","block") );	
-    fldAdd( new TFld("PERIOD",I18N("Calc period (ms)"),TFld::Dec,FLD_NOFLG,"5","1000","0;10000") );
-    fldAdd( new TFld("PRIOR",I18N("Calc task priority"),TFld::Dec,FLD_NOFLG,"2","0","0;100") );
-    fldAdd( new TFld("PER_DB",I18N("Sync db period (s)"),TFld::Dec,FLD_PREV,"5","0","0;3600") );
-    fldAdd( new TFld("ITER",I18N("Iteration number into calc period"),TFld::Dec,FLD_NOFLG,"2","1","0;99") );
+    fldAdd( new TFld("PRM_BD",_("Parameters table"),TFld::String,TFld::NoFlag,"30","system") );
+    fldAdd( new TFld("BLOCK_SH",_("Block's table"),TFld::String,TFld::NoFlag,"30","block") );	
+    fldAdd( new TFld("PERIOD",_("Calc period (ms)"),TFld::Integer,TFld::NoFlag,"5","1000","0;10000") );
+    fldAdd( new TFld("PRIOR",_("Calc task priority"),TFld::Integer,TFld::NoFlag,"2","0","0;100") );
+    fldAdd( new TFld("PER_DB",_("Sync db period (s)"),TFld::Integer,TCfg::Prevent,"5","0","0;3600") );
+    fldAdd( new TFld("ITER",_("Iteration number into calc period"),TFld::Integer,TFld::NoFlag,"2","1","0;99") );
     
     //Add parameter types
-    int t_prm = tpParmAdd("std","PRM_BD",I18N("Standard"));
-    tpPrmAt(t_prm).fldAdd( new TFld("BLK",I18N("Block"),TFld::String,FLD_NOVAL,"10") );
-    tpPrmAt(t_prm).fldAdd( new TFld("IO",I18N("IOs(Sep - ';')"),TFld::String,FLD_NOVAL,"50") );
+    int t_prm = tpParmAdd("std","PRM_BD",_("Standard"));
+    tpPrmAt(t_prm).fldAdd( new TFld("BLK",_("Block"),TFld::String,TCfg::NoVal,"10") );
+    tpPrmAt(t_prm).fldAdd( new TFld("IO",_("IOs(Sep - ';')"),TFld::String,TCfg::NoVal,"50") );
     
     //Blok's db structure
-    blk_el.fldAdd( new TFld("ID",Mess->I18N("ID"),TFld::String,FLD_KEY,"10") );
-    blk_el.fldAdd( new TFld("NAME",Mess->I18N("Name"),TFld::String,FLD_NOFLG,"20") );
-    blk_el.fldAdd( new TFld("DESCR",Mess->I18N("Description"),TFld::String,FLD_NOFLG,"100") );
-    blk_el.fldAdd( new TFld("FUNC",Mess->I18N("Function"),TFld::String,FLD_NOFLG,"75") );
-    blk_el.fldAdd( new TFld("EN",Mess->I18N("To enable"),TFld::Bool,FLD_NOFLG,"1","false") );
-    blk_el.fldAdd( new TFld("PROC",Mess->I18N("To process"),TFld::Bool,FLD_NOFLG,"1","false") );
+    blk_el.fldAdd( new TFld("ID",_("ID"),TFld::String,TCfg::Key,"10") );
+    blk_el.fldAdd( new TFld("NAME",_("Name"),TFld::String,TFld::NoFlag,"20") );
+    blk_el.fldAdd( new TFld("DESCR",_("Description"),TFld::String,TFld::NoFlag,"100") );
+    blk_el.fldAdd( new TFld("FUNC",_("Function"),TFld::String,TFld::NoFlag,"75") );
+    blk_el.fldAdd( new TFld("EN",_("To enable"),TFld::Boolean,TFld::NoFlag,"1","0") );
+    blk_el.fldAdd( new TFld("PROC",_("To process"),TFld::Boolean,TFld::NoFlag,"1","0") );
     
     //IO blok's db structure
-    blkio_el.fldAdd( new TFld("BLK_ID",Mess->I18N("Blok's ID"),TFld::String,FLD_KEY,"10") );
-    blkio_el.fldAdd( new TFld("ID",Mess->I18N("IO ID"),TFld::String,FLD_KEY,"10") );
-    blkio_el.fldAdd( new TFld("TLNK",Mess->I18N("Link's type"),TFld::Dec,FLD_NOFLG,"2") );
-    blkio_el.fldAdd( new TFld("LNK",Mess->I18N("Link"),TFld::String,FLD_NOFLG,"50") );
-    blkio_el.fldAdd( new TFld("VAL",Mess->I18N("Link's value"),TFld::String,FLD_NOFLG,"20") );
+    blkio_el.fldAdd( new TFld("BLK_ID",_("Blok's ID"),TFld::String,TCfg::Key,"10") );
+    blkio_el.fldAdd( new TFld("ID",_("IO ID"),TFld::String,TCfg::Key,"10") );
+    blkio_el.fldAdd( new TFld("TLNK",_("Link's type"),TFld::Integer,TFld::NoFlag,"2") );
+    blkio_el.fldAdd( new TFld("LNK",_("Link"),TFld::String,TFld::NoFlag,"50") );
+    blkio_el.fldAdd( new TFld("VAL",_("Link's value"),TFld::String,TFld::NoFlag,"20") );
 }
 
 void TipContr::preDisable(int flag)
@@ -202,32 +188,98 @@ void TipContr::saveBD()
 
 }
 
-void TipContr::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
+void TipContr::copy( const string &src, const string &dst )
 {
-    if( cmd==TCntrNode::Info )
+    string s_el = TSYS::strSepParse(src,1,'.');
+    string d_el = TSYS::strSepParse(dst,1,'.');
+    string t_el;
+    vector<string>  el_ls, tmp_ls;
+    //- Check parameters -
+    //if( !s_el.empty() && d_el.empty() ) d_el = s_el;
+    if( !s_el.empty() && !d_el.empty() && ( s_el.size() <= 4 || d_el.size() <= 4 ||
+	    (s_el.substr(0,4) != "prm_" && s_el.substr(0,4) != "blk_") || 
+	    s_el.substr(0,4) != d_el.substr(0,4) ) )
+        throw TError(nodePath().c_str(),_("Parameters record error."));	
+
+    //- Connect to source and destination DB -
+    AutoHD<Contr> s_cntr = at(TSYS::strSepParse(src,0,'.'));
+    AutoHD<Contr> d_cntr = at(TSYS::strSepParse(dst,0,'.'));
+    //- Controllers list prepare -
+    if( s_el.empty() || s_el == "blk_*" )
     {
-	TTipDAQ::cntrCmd_( a_path, opt, cmd );
-	ctrMkNode("fld",opt,-1,a_path.c_str(),"/help/g_help",Mess->I18N("Options help"),0440,0,0,3,"tp","str","cols","90","rows","5");
+	s_cntr.at().blkList(tmp_ls);
+	for( int i_el = 0; i_el < tmp_ls.size(); i_el++ )
+	    el_ls.push_back("blk_"+tmp_ls[i_el]);
     }
-    else if( cmd==TCntrNode::Get )
+    if( s_el.empty() || s_el == "prm_*" )
     {
-	if( a_path == "/help/g_help" )	ctrSetS( opt, optDescr() );
-	else TTipDAQ::cntrCmd_( a_path, opt, cmd );
+	s_cntr.at().list(tmp_ls);
+	for( int i_el = 0; i_el < tmp_ls.size(); i_el++ )
+	    el_ls.push_back("prm_"+tmp_ls[i_el]);
     }
-    else if( cmd==TCntrNode::Set )
-	TTipDAQ::cntrCmd_( a_path, opt, cmd );
+    if( el_ls.empty() ) el_ls.push_back(s_el);
+    if( el_ls.size() > 1 && !d_el.empty() )
+        throw TError(nodePath().c_str(),_("Copy several elements to one element not allow."));
+    //- Controllers list process -
+    for( int i_l = 0; i_l < el_ls.size(); i_l++ )
+    {
+	s_el = el_ls[i_l].substr(4);
+	t_el = d_el.empty()?s_el:d_el.substr(4);
+	if( el_ls[i_l].substr(0,4) == "blk_" )
+	{
+	    if( !d_cntr.at().blkPresent(t_el) )	d_cntr.at().blkAdd(t_el);
+	    d_cntr.at().blkAt(t_el).at() = s_cntr.at().blkAt(s_el).at();
+	}
+	else if( el_ls[i_l].substr(0,4) == "prm_" )
+	{
+	    if( !d_cntr.at().present(t_el) )	d_cntr.at().add(t_el,tpPrmToId("std"));
+	    d_cntr.at().at(t_el).at() = s_cntr.at().at(s_el).at();
+	}
+    }
+} 
+
+void TipContr::cntrCmdProc( XMLNode *opt )
+{
+    //Get page info
+    if( opt->name() == "info" )
+    {
+        TTipDAQ::cntrCmdProc(opt);
+	if( ctrMkNode("area",opt,1,"/copy",_("Copy")) && 
+		ctrMkNode("comm",opt,-1,"/copy/copy",_("Copy controller/block/parameter"),0660) )
+	{
+	    ctrMkNode("fld",opt,-1,"/copy/copy/src",_("Source"),0660,"root","root",3,"tp","str","dest","sel_ed","select","/copy/cntrls");
+    	    ctrMkNode("fld",opt,-1,"/copy/copy/dst",_("Destination"),0660,"root","root",3,"tp","str","dest","sel_ed","select","/copy/cntrls");
+	}		
+	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","5");
+    }
+    //Process command to page
+    string a_path = opt->attr("path");
+    if( a_path == "/copy/copy" && ctrChkNode(opt,"set",0440) )
+	copy(ctrId(opt,"src")->text(),ctrId(opt,"dst")->text());
+    else if( a_path == "/copy/cntrls" && ctrChkNode(opt) )
+    {
+	vector<string> lst;
+        list(lst);
+    	for( unsigned i_a=0; i_a < lst.size(); i_a++ )
+	{
+            opt->childAdd("el")->setText(lst[i_a]);
+            opt->childAdd("el")->setText(lst[i_a]+".prm_*");
+            opt->childAdd("el")->setText(lst[i_a]+".blk_*");
+	}
+    }    
+    else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) ) opt->setText(optDescr());
+    else TTipDAQ::cntrCmdProc(opt);
 }
 
-//======================================================================
-//==== Contr 
-//======================================================================
+//************************************************
+//* Contr - Blocks and parameters container      *
+//************************************************ 
 Contr::Contr( string name_c, const string &daq_db, ::TElem *cfgelem) :
     ::TController(name_c, daq_db, cfgelem), prc_st(false), endrun_req(false), sync_st(false), tm_calc(0.0),
     m_per(cfg("PERIOD").getId()), m_prior(cfg("PRIOR").getId()), m_iter(cfg("ITER").getId()), m_dbper(cfg("PER_DB").getId())
 {
-    cfg("PRM_BD").setS(name_c+"_prm");
-    cfg("BLOCK_SH").setS(name_c+"_blocks");
-    hd_res = ResAlloc::resCreate();
+    cfg("PRM_BD").setS("BlckCalcPrm_"+name_c);
+    cfg("BLOCK_SH").setS("BlckCalcBlcks_"+name_c);
     m_bl = grpAdd("blk_");
     
     //Create sync DB timer
@@ -242,7 +294,6 @@ Contr::Contr( string name_c, const string &daq_db, ::TElem *cfgelem) :
 Contr::~Contr()
 {
     timer_delete(sncDBTm);
-    ResAlloc::resDelete(hd_res);
 }
 
 void Contr::postDisable(int flag)
@@ -253,7 +304,7 @@ void Contr::postDisable(int flag)
         if( flag )
         {
 	    //Delete parameter's tables
-	    string wbd = genBD()+"."+cfg("BLOCK_SH").getS();
+	    string wbd = DB()+"."+cfg("BLOCK_SH").getS();
 	    SYS->db().at().open(wbd);	
 	    SYS->db().at().close(wbd,true);
 	    
@@ -262,7 +313,7 @@ void Contr::postDisable(int flag)
 	    SYS->db().at().close(wbd,true);
 	}
     }catch(TError err)
-    { Mess->put(nodePath().c_str(),TMess::Error,"%s",err.mess.c_str()); }	    
+    { mess_err(nodePath().c_str(),"%s",err.mess.c_str()); }	    
     
     TController::postDisable(flag);
 }
@@ -271,40 +322,58 @@ void Contr::load( )
 {
     TController::load( );
     
-    if( en_st )	loadV();
+    loadV();
 }
 
 void Contr::save( )
 {
     TController::save();
 
-    if( en_st ) saveV();
+    saveV();
 }
 
-void Contr::start( )
+void Contr::enable_( )
+{
+    //Enable blocks 
+    vector<string> lst;
+    blkList(lst);
+    for( int i_l = 0; i_l < lst.size(); i_l++ )
+        if( blkAt(lst[i_l]).at().toEnable() )
+	try{ blkAt(lst[i_l]).at().setEnable(true); }
+	catch(TError err)
+	{ 
+	    mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); 
+	    mess_warning(nodePath().c_str(),_("Enable block <%s> error."),lst[i_l].c_str());
+	}
+}
+
+void Contr::disable_( )
+{
+    //Disable blocks
+    vector<string> lst;
+    blkList(lst);
+    for( int i_l = 0; i_l < lst.size(); i_l++ )
+        if( blkAt(lst[i_l]).at().enable() )
+        try{ blkAt(lst[i_l]).at().setEnable(false); }
+	    catch(TError err)
+	    { 
+		mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); 
+		mess_warning(nodePath().c_str(),_("Enable block <%s> error."),lst[i_l].c_str());
+	    }
+}
+
+void Contr::start_( )
 {   
-    TController::start();
-    
-    if( run_st ) return;
-    
     //Make process all bloks
     vector<string> lst;
     blkList(lst);
     for( int i_l = 0; i_l < lst.size(); i_l++ )
-	if( blkAt(lst[i_l]).at().toEnable() )
-	try{ blkAt(lst[i_l]).at().enable(true); }
+        if( blkAt(lst[i_l]).at().enable() && blkAt(lst[i_l]).at().toProcess() )	
+	try{ blkAt(lst[i_l]).at().setProcess(true); }
 	catch(TError err)
 	{ 
-	    Mess->put(err.cat.c_str(),TMess::Warning,"%s",err.mess.c_str()); 
-	    Mess->put(nodePath().c_str(),TMess::Warning,mod->I18N("Enable block <%s> error."),lst[i_l].c_str());
-	}
-    for( int i_l = 0; i_l < lst.size(); i_l++ )
-        if( blkAt(lst[i_l]).at().toProcess() )	
-	try{ blkAt(lst[i_l]).at().process(true); }
-	catch(TError err)
-	{ 
-	    Mess->put(err.cat.c_str(),TMess::Warning,"%s",err.mess.c_str()); 
-	    Mess->put(nodePath().c_str(),TMess::Warning,mod->I18N("Process block <%s> error."),lst[i_l].c_str());
+	    mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); 
+	    mess_warning(nodePath().c_str(),_("Process block <%s> error."),lst[i_l].c_str());
 	}
     
     //Start the request and calc data task
@@ -322,62 +391,25 @@ void Contr::start( )
         pthread_create(&calcPthr,&pthr_attr,Contr::Task,this);
         pthread_attr_destroy(&pthr_attr);
         if( TSYS::eventWait(prc_st, true, nodePath()+"start",5) )
-            throw TError(nodePath().c_str(),mod->I18N("Acquisition task no started!"));
+            throw TError(nodePath().c_str(),_("Acquisition task no started!"));
     }	    
 
     //Start interval timer for periodic thread creating of DB syncing
     struct itimerspec itval;
     itval.it_interval.tv_sec = itval.it_value.tv_sec = m_dbper;
     itval.it_interval.tv_nsec = itval.it_value.tv_nsec = 0;
-    timer_settime(sncDBTm, 0, &itval, NULL);		    
-
-    //Enable parameters
-    vector<string> prm_list;
-    list(prm_list);
-    for( int i_prm = 0; i_prm < prm_list.size(); i_prm++ )
-        if( at(prm_list[i_prm]).at().toEnable() )
-	try{ at(prm_list[i_prm]).at().enable(); }
-	catch(TError err)
-	{ 
-	    Mess->put(err.cat.c_str(),TMess::Warning,"%s",err.mess.c_str()); 
-	    Mess->put(nodePath().c_str(),TMess::Warning,mod->I18N("Enable parameter <%s> error."),prm_list[i_prm].c_str());
-	}
-
-    run_st = true;	
+    timer_settime(sncDBTm, 0, &itval, NULL);
 }
 
-void Contr::stop( )
+void Contr::stop_( )
 {  
-    TController::stop();
-
-    if( !run_st ) return;    
-
-    //Disable parameters
-    vector<string> prm_list;
-    list(prm_list);
-    for( int i_prm = 0; i_prm < prm_list.size(); i_prm++ )
-        if( at(prm_list[i_prm]).at().enableStat() )
-            try{ at(prm_list[i_prm]).at().disable(); }
-            catch(TError err)
-	    { 
-		Mess->put(err.cat.c_str(),TMess::Warning,"%s",err.mess.c_str());
-		Mess->put(nodePath().c_str(),TMess::Warning,mod->I18N("Disable parameter <%s> error."),prm_list[i_prm].c_str());
-	    }
-
-    //Make deprocess all bloks
-    vector<string> lst;
-    blkList(lst);
-    for( int i_l = 0; i_l < lst.size(); i_l++ )
-        if( blkAt(lst[i_l]).at().process() )
-    	    blkAt(lst[i_l]).at().process(false);					
-
     //Stop the request and calc data task
     if( prc_st )
     {
 	endrun_req = true;
         pthread_kill( calcPthr, SIGALRM );
         if( TSYS::eventWait(prc_st,false,nodePath()+"stop",5) )
-            throw TError(nodePath().c_str(),mod->I18N("Acquisition task no stoped!"));
+            throw TError(nodePath().c_str(),_("Acquisition task no stoped!"));
         pthread_join( calcPthr, NULL );
     }
 
@@ -387,40 +419,21 @@ void Contr::stop( )
 	itval.it_value.tv_sec = itval.it_value.tv_nsec = 0;
     timer_settime(sncDBTm, 0, &itval, NULL);
     if( TSYS::eventWait( sync_st, false, nodePath()+"sync_stop",5) )
-        throw TError(nodePath().c_str(),mod->I18N("Controller sync DB no stoped!"));	
+        throw TError(nodePath().c_str(),_("Controller sync DB no stoped!"));
 	
-    run_st = false;
+    //Make deprocess all blocks
+    vector<string> lst;
+    blkList(lst);
+    for( int i_l = 0; i_l < lst.size(); i_l++ )
+        if( blkAt(lst[i_l]).at().process() )
+    	    blkAt(lst[i_l]).at().setProcess(false);
 } 
 
-void Contr::enable_( )
-{
-    try
-    { 
-	loadV( );
-	 
-	vector<string> lst;
-        blkList(lst);
-        for( int i_l = 0; i_l < lst.size(); i_l++ )
-            if( blkAt(lst[i_l]).at().toEnable() )
-                blkAt(lst[i_l]).at().enable(true);
-    }
-    catch(TError err)
-    { 
-	Mess->put(err.cat.c_str(),TMess::Warning,"%s",err.mess.c_str());
-	Mess->put(nodePath().c_str(),TMess::Warning,mod->I18N("Error enable controller."));
-    }
-}
-
-void Contr::disable_( )
-{
-    try{ freeV( ); }
-    catch(TError err) { enable_( ); throw; }
-}
-
 void Contr::loadV( )
 {
-    TConfig c_el(&mod->blockE());	    
-    string bd = genBD()+"."+cfg("BLOCK_SH").getS();
+    TConfig c_el(&mod->blockE());
+    c_el.cfgViewAll(false);	    
+    string bd = DB()+"."+cfg("BLOCK_SH").getS();
     
     int fld_cnt = 0;
     while( SYS->db().at().dataSeek(bd,mod->nodePath()+cfg("BLOCK_SH").getS(),fld_cnt++,c_el) )
@@ -456,52 +469,60 @@ void Contr::freeV( )
     for( int i_l = 0; i_l < lst.size(); i_l++ )    
     {
 	try{ blkDel(lst[i_l]); }
-	catch(TError){ throw TError(nodePath().c_str(),mod->I18N("Can't delete block <%s>."),lst[i_l].c_str()); }
+	catch(TError){ throw TError(nodePath().c_str(),_("Can't delete block <%s>."),lst[i_l].c_str()); }
     }
 }
 
 void *Contr::Task( void *icontr )
 {
-    long long work_tm;
+    long long work_tm, last_tm = 0;
     struct timespec get_tm;
     Contr &cntr = *(Contr *)icontr;    
     
     cntr.endrun_req = false;
     cntr.prc_st = true;
     
-    while(!cntr.endrun_req)
+    bool is_start = true;
+    bool is_stop  = false;
+    
+    while(true)
     {   
 	//Check calk time
         unsigned long long t_cnt = SYS->shrtCnt();
 	
-	ResAlloc::resRequestR(cntr.hd_res);
+	cntr.hd_res.resRequestR( );
 	for(unsigned i_it = 0; i_it < cntr.m_iter; i_it++)
 	    for(unsigned i_blk = 0; i_blk < cntr.clc_blks.size(); i_blk++)
 	    {
-	        try{ cntr.clc_blks[i_blk].at().calc(); }
+	        try{ cntr.clc_blks[i_blk].at().calc(is_start,is_stop); }
 	        catch(TError err)
 	        { 
-	    	    Mess->put(err.cat.c_str(),TMess::Error,"%s",err.mess.c_str());
+	    	    mess_err(err.cat.c_str(),"%s",err.mess.c_str());
 		    string blck = cntr.clc_blks[i_blk].at().id();
-		    Mess->put(cntr.nodePath().c_str(),TMess::Error,mod->I18N("Block <%s> calc error."),blck.c_str());
+		    mess_err(cntr.nodePath().c_str(),_("Block <%s> calc error."),blck.c_str());
 		    if( cntr.clc_blks[i_blk].at().errCnt() < 10 ) continue;
-		    ResAlloc::resReleaseR(cntr.hd_res);
-		    Mess->put(cntr.nodePath().c_str(),TMess::Error,mod->I18N("Block <%s> stoped."),blck.c_str());
-		    cntr.blkAt(blck).at().process(false);			
-		    ResAlloc::resRequestR(cntr.hd_res);
+		    cntr.hd_res.resReleaseR( );
+		    mess_err(cntr.nodePath().c_str(),_("Block <%s> stoped."),blck.c_str());
+		    cntr.blkAt(blck).at().setProcess(false);			
+		    cntr.hd_res.resRequestR( );
 		}
 	    }
-	ResAlloc::resReleaseR(cntr.hd_res);	
+	cntr.hd_res.resReleaseR( );
+	
+	cntr.tm_calc = 1.0e6*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk());
+	
+	if(is_stop) break;
 		
 	//Calc next work time and sleep
         clock_gettime(CLOCK_REALTIME,&get_tm);
-        work_tm = (long long)get_tm.tv_sec*1000000000+get_tm.tv_nsec;
-        work_tm = (work_tm/(cntr.m_per*1000000) + 1)*cntr.m_per*1000000;
+        work_tm = (((long long)get_tm.tv_sec*1000000000+get_tm.tv_nsec)/((long long)cntr.m_per*1000000) + 1)*(long long)cntr.m_per*1000000;
+	if(work_tm == last_tm)	work_tm+=(long long)cntr.m_per*1000000;	//Fix early call
+	last_tm = work_tm;
         get_tm.tv_sec = work_tm/1000000000; get_tm.tv_nsec = work_tm%1000000000;
-	
-	cntr.tm_calc = 1.0e6*((double)(SYS->shrtCnt()-t_cnt))/((double)SYS->sysClk());	
-	
 	clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&get_tm,NULL);
+	
+	if(cntr.endrun_req) is_stop = true;
+        is_start = false;
     }
     
     cntr.prc_st = false;
@@ -518,8 +539,8 @@ void Contr::TaskDBSync(union sigval obj)
     try{ cntr->saveV( ); }
     catch(TError err) 
     { 
-	Mess->put(err.cat.c_str(),TMess::Error,"%s",err.mess.c_str() ); 
-	Mess->put(cntr->nodePath().c_str(),TMess::Error,mod->I18N("Block save error."));
+	mess_err(err.cat.c_str(),"%s",err.mess.c_str() ); 
+	mess_err(cntr->nodePath().c_str(),_("Block save error."));
     }
     
     cntr->sync_st = false;
@@ -550,30 +571,6 @@ void Contr::blkProc( const string & id, bool val )
 	clc_blks.erase(clc_blks.begin()+i_blk);
 }	
 
-void Contr::copyBlock( const string &from_id, const string &cntr_id, const string &to_id, const string &to_name )
-{
-    string contr = cntr_id;
-    string toid = to_id;
-    string toname = to_name;
-    
-    if( !blkPresent(from_id) )
-        throw TError(nodePath().c_str(),mod->I18N("Block <%s> no present."),from_id.c_str());
-
-    if( !contr.size() )	contr = id();
-    if( !toid.size() )  toid  = blkAt(from_id).at().id();
-    if( !toname.size() )toname = blkAt(from_id).at().name();
-    
-    if( !mod->present(contr) )
-        throw TError(nodePath().c_str(),mod->I18N("Controller <%s> no present."),contr.c_str());
-    if( mod->at(contr).at().blkPresent(toid) )
-        throw TError(nodePath().c_str(),mod->I18N("Block <%s:%s> already present."),contr.c_str(),toid.c_str());
-	
-    //Make new function
-    mod->at(contr).at().blkAdd(toid);
-    mod->at(contr).at().blkAt(toid).at() = blkAt(from_id).at();
-    mod->at(contr).at().blkAt(toid).at().name(to_name);
-}
-
 bool Contr::cfgChange( TCfg &cfg )
 {
     if( startStat() )
@@ -589,71 +586,46 @@ bool Contr::cfgChange( TCfg &cfg )
     return true;
 }
 
-void Contr::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
+void Contr::cntrCmdProc( XMLNode *opt )
 {
-    vector<string> lst;
-    
-    if( cmd==TCntrNode::Info )
+    //Get page info
+    if( opt->name() == "info" )
     {
-	TController::cntrCmd_( a_path, opt, cmd );
-
-	ctrMkNode("area",opt,-1,a_path.c_str(),"/scheme",owner().I18N("Blocks scheme"));
-	ctrMkNode("fld",opt,-1,a_path.c_str(),"/scheme/ctm",owner().I18N("Calk time (usek)"),0444,0,0,1,"tp","real");
-	ctrMkNode("list",opt,-1,a_path.c_str(),"/scheme/sch",owner().I18N("Blocks"),0664,0,0,4,"tp","br","idm","1","s_com","add,del","br_pref","blk_");
-	ctrMkNode("comm",opt,-1,a_path.c_str(),"/scheme/copy",mod->I18N("Copy block"),0440);
-	ctrMkNode("fld",opt,-1,a_path.c_str(),"/scheme/copy/blk",mod->I18N("Block"),0660,0,0,4,"tp","str","idm","1","dest","select","select","/scheme/ls_blck");
-        ctrMkNode("fld",opt,-1,a_path.c_str(),"/scheme/copy/cntr",mod->I18N("To controller"),0660,0,0,4,"tp","str","idm","1","dest","select","select","/scheme/ls_cntr");
-        ctrMkNode("fld",opt,-1,a_path.c_str(),"/scheme/copy/id",mod->I18N("Name as"),0660,0,0,2,"tp","str","len","10");
-        ctrMkNode("fld",opt,-1,a_path.c_str(),"/scheme/copy/nm","",0660,0,0,1,"tp","str");
-    }
-    else if( cmd==TCntrNode::Get )
-    {
-	if( a_path == "/scheme/ctm" )  ctrSetR( opt, tm_calc );
-	else if( a_path == "/scheme/sch" )
+        TController::cntrCmdProc(opt);
+	ctrMkNode("grp",opt,-1,"/br/blk_",_("Block"),0440,"root","root",1,"list","/scheme/sch");
+	if(ctrMkNode("area",opt,-1,"/scheme",_("Blocks scheme")))
 	{
-	    vector<string> list_el;
-	    blkList(list_el);
-	    opt->childClean();
-	    for( unsigned i_f=0; i_f < list_el.size(); i_f++ )
-		ctrSetS( opt, blkAt(list_el[i_f]).at().name(), list_el[i_f].c_str() );
+	    ctrMkNode("fld",opt,-1,"/scheme/ctm",_("Calk time (usek)"),0444,"root","root",1,"tp","real");
+	    ctrMkNode("list",opt,-1,"/scheme/sch",_("Blocks"),0664,"root","root",4,"tp","br","idm","1","s_com","add,del","br_pref","blk_");
 	}
-	else if( a_path == "/scheme/ls_blck" )
-        {
-            blkList(lst);
-            opt->childClean();
-    	    for( unsigned i_f=0; i_f < lst.size(); i_f++ )
-                ctrSetS( opt, blkAt(lst[i_f]).at().name(), lst[i_f].c_str() );
-        }
-	else if( a_path == "/scheme/ls_cntr" )
-        {
-            opt->childClean();
-            ctrSetS( opt, "", "" );
-            mod->list(lst);
-    	    for( unsigned i_a=0; i_a < lst.size(); i_a++ )
-                ctrSetS( opt, mod->at(lst[i_a]).at().name(), lst[i_a].c_str() );
-        }	
-	else TController::cntrCmd_( a_path, opt, cmd );
+        return;
     }
-    else if( cmd==TCntrNode::Set )
+    //Process command to page
+    string a_path = opt->attr("path");
+    if( a_path == "/scheme/ctm" && ctrChkNode(opt) )	opt->setText(TSYS::real2str(tm_calc,6));
+    else if( a_path == "/scheme/sch" )
     {
-	if( a_path == "/scheme/sch" )
+	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
 	{
-	    if( opt->name() == "add" )		
-	    {
-		blkAdd(opt->attr("id"));
-		blkAt(opt->attr("id")).at().name(opt->text());
-	    }
-	    else if( opt->name() == "del" )	chldDel(m_bl,opt->attr("id"),-1,1);
+	    vector<string> lst;
+	    blkList(lst);
+	    for( unsigned i_f=0; i_f < lst.size(); i_f++ )
+		opt->childAdd("el")->setAttr("id",lst[i_f])->setText(blkAt(lst[i_f]).at().name());
 	}
-	else if( a_path == "/scheme/copy" )
-            copyBlock(ctrGetS(ctrId(opt,"blk")),ctrGetS(ctrId(opt,"cntr")), ctrGetS(ctrId(opt,"id")), ctrGetS(ctrId(opt,"nm")));
-	else TController::cntrCmd_( a_path, opt, cmd );	
+	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )		
+	{
+	    blkAdd(opt->attr("id"));
+	    blkAt(opt->attr("id")).at().setName(opt->text());
+	}
+	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	chldDel(m_bl,opt->attr("id"),-1,1);
     }
-}
+    else TController::cntrCmdProc(opt);
+}	
 
-//======================================================================
-//==== Prm 
-//======================================================================
+//************************************************
+//* Prm - parameters for access to data          *
+//*       of calced blocks                       *
+//************************************************  
 Prm::Prm( string name, TTipParam *tp_prm ) : 
     TParamContr(name,tp_prm), v_el(name)
 {
@@ -665,9 +637,9 @@ Prm::~Prm()
     nodeDelAll();
 }
 
-void Prm::postEnable()
+void Prm::postEnable( int flag )
 {
-    TParamContr::postEnable();
+    TParamContr::postEnable( flag );
     if(!vlElemPresent(&v_el))	vlElemAtt(&v_el);
 }
 
@@ -678,27 +650,26 @@ void Prm::enable()
     m_blck = cfg("BLK").getS();
     
     if( !((Contr &)owner()).blkPresent(m_blck) )
-        throw TError(nodePath().c_str(),mod->I18N("Block not connected."));
+        throw TError(nodePath().c_str(),_("Block not connected."));
     
     //Init elements
-    int io_cnt = 0;
-    while(TSYS::strSepParse(cfg("IO").getS(),io_cnt,';').size())
+    string mio;
+    for( int io_off = 0; (mio=TSYS::strSepParse(cfg("IO").getS(),0,';',&io_off)).size(); )
     {
-	string mio = TSYS::strSepParse(cfg("IO").getS(),io_cnt,';');
-        unsigned char flg = FLD_DWR|FLD_DRD;
+        unsigned flg = TVal::DirWrite|TVal::DirRead;
         TFld::Type    tp  = TFld::String;
         int           io_id = ((Contr &)owner()).blkAt(m_blck).at().ioId(mio);
         if(io_id >= 0)
 	{
 	    //if( ((Contr &)owner()).blkAt(m_blck).at().ioMode(io_id) != IO::Input )
-	    //	flg |= FLD_NWR;
+	    //	flg |= Fld::NoWrite;
 	    switch( ((Contr &)owner()).blkAt(m_blck).at().ioType(io_id) )
 	    {
 		case IO::String: 	tp = TFld::String; 	break;
-		case IO::Integer:	tp = TFld::Dec;		break;
+		case IO::Integer:	tp = TFld::Integer;	break;
 		case IO::Real:		tp = TFld::Real;	break;
-		case IO::Boolean:	tp = TFld::Bool;	break;    
-	    }    
+		case IO::Boolean:	tp = TFld::Boolean;	break;
+	    }
 	    if( !v_el.fldPresent(mio) || 
 		v_el.fldAt(v_el.fldId(mio)).type() != tp || 
 		v_el.fldAt(v_el.fldId(mio)).flg() != flg )
@@ -707,17 +678,15 @@ void Prm::enable()
 		v_el.fldAdd( new TFld(mio.c_str(),((Contr &)owner()).blkAt(m_blck).at().func()->io(io_id)->name().c_str(),tp,flg) );
     	    }
 	}
-	io_cnt++;
     }
     
     //Check and delete no used fields
     for(int i_fld = 0; i_fld < v_el.fldSize(); i_fld++)
     {
         string fel;
-        int io_cnt = 0;
-        while( (fel = TSYS::strSepParse(cfg("IO").getS(),io_cnt++,';')).size() )
+        for( int io_off = 0; (fel=TSYS::strSepParse(cfg("IO").getS(),0,';',&io_off)).size(); )
     	    if( fel == v_el.fldAt(i_fld).name() ) break;
-        if( !fel.size() )
+        if( fel.empty() )
         {
             v_el.fldDel(i_fld);
             i_fld--;
@@ -748,13 +717,13 @@ void Prm::vlSet( TVal &val )
 		case TFld::String: 		    
 		    ((Contr &)owner()).blkAt(m_blck).at().setS(io_id,val.getS(0,true));
 		    break;
-		case TFld::Dec:
+		case TFld::Integer:
 		    ((Contr &)owner()).blkAt(m_blck).at().setI(io_id,val.getI(0,true));
 		    break;
 		case TFld::Real:
 		    ((Contr &)owner()).blkAt(m_blck).at().setR(io_id,val.getR(0,true));
 		    break;
-		case TFld::Bool:
+		case TFld::Boolean:
 		    ((Contr &)owner()).blkAt(m_blck).at().setB(io_id,val.getB(0,true));
 		    break;
 	    }
@@ -767,15 +736,15 @@ void Prm::vlGet( TVal &val )
     if( val.name() == "err" )
     {    
 	if( !owner().startStat() )             
-	    val.setS(mod->I18N("2:Controller stoped"),0,true);
+	    val.setS(_("2:Controller stoped"),0,true);
 	else if( !enableStat() )   		   
-	    val.setS(mod->I18N("1:Parameter disabled"),0,true);
+	    val.setS(_("1:Parameter disabled"),0,true);
 	else if( !owner().blkPresent(m_blck) ) 
-	    val.setS(mod->I18N("3:Block no present"),0,true);
+	    val.setS(_("3:Block no present"),0,true);
 	else if( !owner().blkAt(m_blck).at().enable() )
-    	    val.setS(mod->I18N("4:Block disabled"),0,true);
+    	    val.setS(_("4:Block disabled"),0,true);
 	else if( !owner().blkAt(m_blck).at().process() )
-    	    val.setS(mod->I18N("5:Block no process"),0,true);
+    	    val.setS(_("5:Block no process"),0,true);
 	else val.setS("0",0,true);	
 	return;
     }
@@ -790,36 +759,45 @@ void Prm::vlGet( TVal &val )
 	    {
 		case TFld::String: 
 		    val.setS(enableStat()?owner().blkAt(m_blck).at().getS(io_id):EVAL_STR,0,true); break;
-		case TFld::Dec:
+		case TFld::Integer:
 		    val.setI(enableStat()?owner().blkAt(m_blck).at().getI(io_id):EVAL_INT,0,true); break;
 		case TFld::Real:
 		    val.setR(enableStat()?owner().blkAt(m_blck).at().getR(io_id):EVAL_REAL,0,true);break;
-		case TFld::Bool:
+		case TFld::Boolean:
 		    val.setB(enableStat()?owner().blkAt(m_blck).at().getB(io_id):EVAL_BOOL,0,true);break;
 	    }
     }catch(TError err) { disable(); }
 }
 
-void Prm::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
+void Prm::vlArchMake( TVal &val )
 {
-    if( cmd==TCntrNode::Info )
+    if( val.arch().freeStat() ) return;
+	val.arch().at().setSrcMode(TVArchive::ActiveAttr,val.arch().at().srcData());
+    val.arch().at().setPeriod(1000000);
+    val.arch().at().setHardGrid( true );
+    val.arch().at().setHighResTm( false );
+}
+
+void Prm::cntrCmdProc( XMLNode *opt )
+{
+    //- Service commands process -
+    string a_path = opt->attr("path");
+    if( a_path.substr(0,6) == "/serv/" )  { TParamContr::cntrCmdProc(opt); return; }
+
+    //- Get page info -
+    if( opt->name() == "info" )
     {
-        TParamContr::cntrCmd_( a_path, opt, cmd );
-		
-        ctrId(opt,"/prm/cfg/BLK")->attr_("dest","select")->attr_("select","/prm/cfg/BLK_list");
+	TParamContr::cntrCmdProc(opt);
+	ctrMkNode("fld",opt,-1,"/prm/cfg/BLK",cfg("BLK").fld().descr(),0660,"root","root",3,"tp","str","dest","select","select","/prm/cfg/BLK_list");
+	return;
     }
-    else if( cmd==TCntrNode::Get )
+    //- Process command to page -
+    if( a_path == "/prm/cfg/BLK_list" && ctrChkNode(opt) )
     {
-        if( a_path == "/prm/cfg/BLK_list" )
-        {
-            vector<string> list;
-            ((Contr &)owner()).blkList(list);
-            opt->childClean();
-            for( unsigned i_f=0; i_f < list.size(); i_f++ )
-        	ctrSetS( opt, ((Contr &)owner()).blkAt(list[i_f]).at().name(), list[i_f].c_str() );
-	}
-        else TParamContr::cntrCmd_( a_path, opt, cmd );
+        vector<string> list;
+        ((Contr &)owner()).blkList(list);
+        for( unsigned i_f=0; i_f < list.size(); i_f++ )
+    	    opt->childAdd("el")->setAttr("id",list[i_f])->setText(owner().blkAt(list[i_f]).at().name());
     }
-    else if( cmd==TCntrNode::Set )
-	TParamContr::cntrCmd_( a_path, opt, cmd );
+    else TParamContr::cntrCmdProc(opt);
 }
