@@ -1,13 +1,12 @@
 
 //OpenSCADA system module Transport.Sockets file: socket.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -35,22 +34,22 @@
 #include <errno.h>
 
 #include <tsys.h>
-#include <resalloc.h>
 #include <tmess.h>
 #include <tprotocols.h>
 #include <tmodule.h>
 #include "socket.h"
 
-//============ Modul info! =====================================================
+//************************************************
+//* Modul info!                                  *
 #define MOD_ID	    "Sockets"
 #define MOD_NAME    "Sockets"
 #define MOD_TYPE    "Transport"
 #define VER_TYPE    VER_TR
-#define VERSION     "1.2.1"
+#define VERSION     "1.3.0"
 #define AUTORS      "Roman Savochenko"
 #define DESCRIPTION "Allow sockets based transport. Support inet and unix sockets. Inet socket use TCP and UDP protocols."
 #define LICENSE     "GPL"
-//==============================================================================
+//************************************************
 
 Sockets::TTransSock *Sockets::mod;
 
@@ -58,37 +57,23 @@ extern "C"
 {
     TModule::SAt module( int n_mod )
     {
-    	TModule::SAt AtMod;
-
-	if(n_mod==0)
-	{
-	    AtMod.id	= MOD_ID;
-	    AtMod.type  = MOD_TYPE;
-	    AtMod.t_ver = VER_TYPE;
-	}
-	else
-    	    AtMod.id	= "";
-
-    	return( AtMod );
+	if( n_mod==0 )	return TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE);
+    	return TModule::SAt("");
     }
 
     TModule *attach( const TModule::SAt &AtMod, const string &source )
     {
-	Sockets::TTransSock *self_addr = NULL;
-
-	if( AtMod.id == MOD_ID && AtMod.type == MOD_TYPE && AtMod.t_ver == VER_TYPE )
-	    self_addr = Sockets::mod = new Sockets::TTransSock( source );
-
-	return ( self_addr );
+	if( AtMod == TModule::SAt(MOD_ID,MOD_TYPE,VER_TYPE) )
+	    return new Sockets::TTransSock( source );
+	return NULL;
     }
 }
 
 using namespace Sockets;
 
-//==============================================================================
-//== TTransSock ================================================================
-//==============================================================================
-    
+//************************************************
+//* TTransSock                                   *
+//************************************************
 TTransSock::TTransSock( string name ) 
 {
     mId		= MOD_ID;
@@ -99,6 +84,8 @@ TTransSock::TTransSock( string name )
     mDescr  	= DESCRIPTION;
     mLicense   	= LICENSE;
     mSource    	= name;
+
+    mod 	= this;
 }
 
 TTransSock::~TTransSock()
@@ -106,23 +93,23 @@ TTransSock::~TTransSock()
 
 }
 
-void TTransSock::postEnable( )
+void TTransSock::postEnable( int flag )
 {
-    TModule::postEnable( );
+    TModule::postEnable( flag );
     
-    //Add self DB-fields BaseArhMSize
-    if( !((TTransportS &)owner()).inEl().fldPresent("SocketsBufLen") )
-	((TTransportS &)owner()).inEl().fldAdd( new TFld("SocketsBufLen",Mess->I18N("Input socket buffer length (kB)"),TFld::Dec,0,"3","5") );
-    if( !((TTransportS &)owner()).inEl().fldPresent("SocketsMaxQueue") )
-	((TTransportS &)owner()).inEl().fldAdd( new TFld("SocketsMaxQueue",Mess->I18N("Maximum queue of input socket"),TFld::Dec,0,"2","10") );
-    if( !((TTransportS &)owner()).inEl().fldPresent("SocketsMaxClient") )
-	((TTransportS &)owner()).inEl().fldAdd( new TFld("SocketsMaxClient",Mess->I18N("Maximum clients process"),TFld::Dec,0,"2","10") );
+    if( flag&TCntrNode::NodeConnect )
+    {
+	//- Add self DB-fields BaseArhMSize -
+	owner().inEl().fldAdd( new TFld("SocketsBufLen",_("Input socket buffer length (kB)"),TFld::Integer,0,"3","5") );
+	owner().inEl().fldAdd( new TFld("SocketsMaxQueue",_("Maximum queue of input socket"),TFld::Integer,0,"2","10") );
+	owner().inEl().fldAdd( new TFld("SocketsMaxClient",_("Maximum clients process"),TFld::Integer,0,"2","10") );
+    }
 }
 
 string TTransSock::optDescr( )
 {
     char buf[STR_BUF_LEN];
-    snprintf(buf,sizeof(buf),I18N(
+    snprintf(buf,sizeof(buf),_(
 	"======================= The module <%s:%s> options =======================\n"
 	"---------- Parameters of the module section <%s> in config file ----------\n\n"),
 	MOD_TYPE,MOD_ID,nodePath().c_str());
@@ -132,7 +119,7 @@ string TTransSock::optDescr( )
 
 void TTransSock::modLoad( )
 {
-    //========== Load parameters from command line ============
+    //- Load parameters from command line -
     int next_opt;
     char *short_opt="h";
     struct option long_opt[] =
@@ -163,39 +150,35 @@ TTransportOut *TTransSock::Out( const string &name, const string &idb )
     return new TSocketOut(name,idb,&owner().outEl());
 }
 
-//================== Controll functions ========================
-void TTransSock::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
+void TTransSock::cntrCmdProc( XMLNode *opt )
 {
-    if( cmd==TCntrNode::Info )
-    {	
-	TTipTransport::cntrCmd_( a_path, opt, cmd );
-
-	ctrMkNode("fld",opt,-1,a_path.c_str(),"/help/g_help",Mess->I18N("Options help"),0440,0,0,3,"tp","str","cols","90","rows","5");
-    }
-    else if( cmd==TCntrNode::Get )
+    //- Get page info -
+    if( opt->name() == "info" )
     {
-	if( a_path == "/help/g_help" ) 	ctrSetS( opt, optDescr() );       
-    	else TTipTransport::cntrCmd_( a_path, opt, cmd );
+        TTipTransport::cntrCmdProc(opt);
+	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","5");
+	return;
     }
-    else if( cmd==TCntrNode::Set )
-	TTipTransport::cntrCmd_( a_path, opt, cmd );    
-}    
 
-//==============================================================================
-//== TSocketIn =================================================================
-//==============================================================================
+    //- Process command to page -
+    string a_path = opt->attr("path");
+    if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) )   opt->setText(optDescr());
+    else TTipTransport::cntrCmdProc(opt);
+}	        
 
+//************************************************
+//* TSocketIn                                    *
+//************************************************
 TSocketIn::TSocketIn( string name, const string &idb, TElem *el ) : 
     TTransportIn(name,idb,el), cl_free(true), max_queue(cfg("SocketsMaxQueue").getId()), 
     max_fork(cfg("SocketsMaxClient").getId()), buf_len(cfg("SocketsBufLen").getId())
 {
-    sock_res = ResAlloc::resCreate();
+    setAddr("TCP:localhost:10002:0");
 }
 
 TSocketIn::~TSocketIn()
 {
     try{ stop(); }catch(...){ }
-    ResAlloc::resDelete(sock_res);
 }
 
 void TSocketIn::start()
@@ -209,22 +192,24 @@ void TSocketIn::start()
     if( s_type == S_NM_TCP )
     {
     	if( (sock_fd = socket(PF_INET,SOCK_STREAM,0) )== -1 ) 
-    	    throw TError(nodePath().c_str(),mod->I18N("Error create %s socket!"),s_type.c_str());
+    	    throw TError(nodePath().c_str(),_("Error create %s socket!"),s_type.c_str());
+	int vl = 1;
+	setsockopt(sock_fd,SOL_SOCKET,SO_REUSEADDR,&vl,sizeof(int));
 	type = SOCK_TCP;
     }
     else if( s_type == S_NM_UDP )
     {	
 	if( (sock_fd = socket(PF_INET,SOCK_DGRAM,0) )== -1 ) 
-    	    throw TError(nodePath().c_str(),mod->I18N("Error create %s socket!"),s_type.c_str());
+    	    throw TError(nodePath().c_str(),_("Error create %s socket!"),s_type.c_str());
 	type = SOCK_UDP;
     }
     else if( s_type == S_NM_UNIX )
     {
     	if( (sock_fd = socket(PF_UNIX,SOCK_STREAM,0) )== -1) 
-    	    throw TError(nodePath().c_str(),mod->I18N("Error create %s socket!"),s_type.c_str());
+    	    throw TError(nodePath().c_str(),_("Error create %s socket!"),s_type.c_str());
 	type = SOCK_UNIX;
     }
-    else throw TError(nodePath().c_str(),mod->I18N("Socket type <%s> error!"),s_type.c_str());
+    else throw TError(nodePath().c_str(),_("Socket type <%s> error!"),s_type.c_str());
 
     if( type == SOCK_TCP || type == SOCK_UDP )
     {
@@ -239,7 +224,7 @@ void TSocketIn::start()
 	{
 	    loc_host_nm = gethostbyname(host.c_str());
 	    if(loc_host_nm == NULL || loc_host_nm->h_length == 0)
-		throw TError(nodePath().c_str(),mod->I18N("Socket name <%s> error!"),host.c_str());
+		throw TError(nodePath().c_str(),_("Socket name <%s> error!"),host.c_str());
 	    name_in.sin_addr.s_addr = *( (int *) (loc_host_nm->h_addr_list[0]) );
 	}
 	else name_in.sin_addr.s_addr = INADDR_ANY;  
@@ -256,7 +241,7 @@ void TSocketIn::start()
 	    {
 	    	shutdown( sock_fd,SHUT_RDWR );
 		close( sock_fd );
-		throw TError(nodePath().c_str(),mod->I18N("TCP socket no bind to <%s>!"),addr().c_str());
+		throw TError(nodePath().c_str(),_("TCP socket no bind to <%s>!"),addr().c_str());
 	    }
 	    listen(sock_fd,max_queue);
 	}
@@ -272,7 +257,7 @@ void TSocketIn::start()
 	    {
 	    	shutdown( sock_fd,SHUT_RDWR );
 		close( sock_fd );
-		throw TError(nodePath().c_str(),mod->I18N("UDP socket no bind to <%s>!"),addr().c_str());
+		throw TError(nodePath().c_str(),_("UDP socket no bind to <%s>!"),addr().c_str());
 	    }
 	}
     }
@@ -289,7 +274,7 @@ void TSocketIn::start()
 	if( bind(sock_fd,(sockaddr *)&name_un,sizeof(name_un) ) == -1) 
 	{
 	    close( sock_fd );
-	    throw TError(nodePath().c_str(),mod->I18N("UNIX socket no bind to <%s>!"),addr().c_str());
+	    throw TError(nodePath().c_str(),_("UNIX socket no bind to <%s>!"),addr().c_str());
 	}
 	listen(sock_fd,max_queue);
     }    
@@ -299,7 +284,7 @@ void TSocketIn::start()
     pthread_create(&pthr_tsk,&pthr_attr,Task,this);
     pthread_attr_destroy(&pthr_attr);
     if( TSYS::eventWait( run_st, true,nodePath()+"open",5) )
-       	throw TError(nodePath().c_str(),mod->I18N("No open!"));   
+       	throw TError(nodePath().c_str(),_("No open!"));   
 }
 
 void TSocketIn::stop()
@@ -323,9 +308,9 @@ void *TSocketIn::Task(void *sock_in)
     TSocketIn *sock = (TSocketIn *)sock_in;
     AutoHD<TProtocolIn> prot_in;
 
-#if OSC_DEBUG
-    Mess->put(sock->nodePath().c_str(),TMess::Debug,Mess->I18N("Thread <%d> started!"),getpid() );
-#endif	
+//#if OSC_DEBUG
+//    mess_debug(sock->nodePath().c_str(),_("Thread <%d> started!"),gettid());
+//#endif	
     
     pthread_t      th;
     pthread_attr_t pthr_attr;
@@ -364,12 +349,8 @@ void *TSocketIn::Task(void *sock_in)
 			close(sock_fd_CL);
 			continue;
 		    }
-		    SSockIn *s_inf = new SSockIn;
-		    s_inf->s_in    = sock;
-		    s_inf->cl_sock = sock_fd_CL;
-		    s_inf->sender  = inet_ntoa(name_cl.sin_addr);		    
-		    if( pthread_create(&th,&pthr_attr,ClTask,s_inf) < 0)
-    		        Mess->put(sock->nodePath().c_str(),TMess::Error,mod->I18N("Error create pthread!"));
+		    if( pthread_create( &th, &pthr_attr, ClTask, new SSockIn( sock, sock_fd_CL, inet_ntoa(name_cl.sin_addr) ) ) < 0)
+    		        mess_err(sock->nodePath().c_str(),_("Error create pthread!"));
 		}
 	    }
 	    else if( sock->type == SOCK_UNIX )
@@ -382,11 +363,8 @@ void *TSocketIn::Task(void *sock_in)
 			close(sock_fd_CL);
 			continue;
 		    }		    
-		    SSockIn *s_inf = new SSockIn;
-		    s_inf->s_in    = sock;
-		    s_inf->cl_sock = sock_fd_CL;		    
-                    if( pthread_create(&th,&pthr_attr,ClTask,s_inf) < 0 ) 
-		    	Mess->put(sock->nodePath().c_str(),TMess::Error,mod->I18N("Error create pthread!"));
+                    if( pthread_create( &th, &pthr_attr, ClTask, new SSockIn(sock,sock_fd_CL,"") ) < 0 )
+		    	mess_err(sock->nodePath().c_str(),_("Error create pthread!"));
 		}	    
 	    }
 	    else if( sock->type == SOCK_UDP )
@@ -397,14 +375,14 @@ void *TSocketIn::Task(void *sock_in)
 		r_len = recvfrom(sock->sock_fd, buf, sock->buf_len*1000, 0,(sockaddr *)&name_cl, &name_cl_len);
 		if( r_len <= 0 ) continue;
 		req.assign(buf,r_len);
-		Mess->put(sock->nodePath().c_str(),TMess::Info,mod->I18N("Socket receive datagram <%d> from <%s>!"),
-			r_len, inet_ntoa(name_cl.sin_addr) );
+		//mess_debug(sock->nodePath().c_str(),_("Socket receive datagram <%d> from <%s>!"),
+		//	r_len, inet_ntoa(name_cl.sin_addr) );
 		
 		sock->PutMess(sock->sock_fd, req, answ, inet_ntoa(name_cl.sin_addr),prot_in);
 		if( !prot_in.freeStat() ) continue;		
 		    
-		Mess->put(sock->nodePath().c_str(),TMess::Info,mod->I18N("Socket reply datagram <%d> to <%s>!"),
-			answ.size(), inet_ntoa(name_cl.sin_addr) );
+		//mess_debug(sock->nodePath().c_str(),_("Socket reply datagram <%d> to <%s>!"),
+		//	answ.size(), inet_ntoa(name_cl.sin_addr) );
 		sendto(sock->sock_fd,answ.c_str(),answ.size(),0,(sockaddr *)&name_cl, name_cl_len);
 	    }
 	}
@@ -412,28 +390,33 @@ void *TSocketIn::Task(void *sock_in)
     pthread_attr_destroy(&pthr_attr);
     
     if( sock->type == SOCK_UDP ) delete []buf;
-    //Client tasks stop command
+    //- Client tasks stop command -
     sock->endrun_cl = true;
+    ResAlloc res(sock->sock_res,false);
+    //- Find already registry -
+    for( int i_id = 0; i_id < sock->cl_id.size(); i_id++)
+        pthread_kill(sock->cl_id[i_id].cl_id,SIGALRM);
+    res.release();
     TSYS::eventWait( sock->cl_free, true, string(MOD_ID)+": "+sock->id()+" client task is stoping....");
 
     sock->run_st = false;
     
-    return(NULL);
+    return NULL;
 }
 
-void *TSocketIn::ClTask(void *s_inf)
+void *TSocketIn::ClTask( void *s_inf )
 {
     SSockIn *s_in = (SSockIn *)s_inf;    
     
-#if OSC_DEBUG
-    Mess->put(s_in->s_in->nodePath().c_str(),TMess::Debug,Mess->I18N("Thread <%d> started!"),getpid() );
-#endif	
+//#if OSC_DEBUG
+//    mess_debug(s_in->s_in->nodePath().c_str(),_("Thread <%d> started!"),pthread_self() );
+//#endif	
     
-    Mess->put(s_in->s_in->nodePath().c_str(),TMess::Info,mod->I18N("Socket have been connected by <%s>!"),s_in->sender.c_str() );
-    s_in->s_in->RegClient( getpid( ), s_in->cl_sock );
+//    mess_debug(s_in->s_in->nodePath().c_str(),_("Socket have been connected by <%s>!"),s_in->sender.c_str() );
+    s_in->s_in->RegClient( pthread_self(), s_in->cl_sock );
     s_in->s_in->ClSock( *s_in );
-    s_in->s_in->UnregClient( getpid( ) );
-    Mess->put(s_in->s_in->nodePath().c_str(),TMess::Info,mod->I18N("Socket have been disconnected by <%s>!"),s_in->sender.c_str() );
+    s_in->s_in->UnregClient( pthread_self( ) );
+//    mess_debug(s_in->s_in->nodePath().c_str(),_("Socket have been disconnected by <%s>!"),s_in->sender.c_str() );
     delete s_in;
                     
     pthread_exit(NULL);    
@@ -448,62 +431,40 @@ void TSocketIn::ClSock( SSockIn &s_in )
     char    buf[buf_len*1000 + 1];    
     AutoHD<TProtocolIn> prot_in;
     
-    if(mode)
+    do
     {
-    	while( !endrun_cl )
-	{
-    	    tv.tv_sec  = 0;
-    	    tv.tv_usec = STD_WAIT_DELAY*1000;  
-	    FD_ZERO(&rd_fd);
-	    FD_SET(s_in.cl_sock,&rd_fd);		
+        tv.tv_sec  = 0;
+        tv.tv_usec = STD_WAIT_DELAY*1000;
+	FD_ZERO(&rd_fd);
+	FD_SET(s_in.cl_sock,&rd_fd);		
 	    
-	    int kz = select(s_in.cl_sock+1,&rd_fd,NULL,NULL,&tv);
-	    if( kz == 0 || (kz == -1 && errno == EINTR) ) continue;
-	    if( kz < 0) continue;
-	    if( FD_ISSET(s_in.cl_sock, &rd_fd) )
-	    {
-		r_len = read(s_in.cl_sock,buf,buf_len*1000);
-		Mess->put(s_in.s_in->nodePath().c_str(),TMess::Info,mod->I18N("Socket receive of message <%d> from <%s>!"), r_len, s_in.sender.c_str() );
-    		if(r_len <= 0) break;
-    		req.assign(buf,r_len);
-
-		PutMess(s_in.cl_sock,req,answ,s_in.sender,prot_in);
-		if( !prot_in.freeStat() ) continue;		
-    		Mess->put(s_in.s_in->nodePath().c_str(),TMess::Info,mod->I18N("Socket reply of message <%d> to <%s>!"), answ.size(), s_in.sender.c_str() );
-	    	r_len = write(s_in.cl_sock,answ.c_str(),answ.size());   
-	    }
-	}
-    }
-    else
-    {
-	do
+	int kz = select(s_in.cl_sock+1,&rd_fd,NULL,NULL,&tv);
+	if( kz == 0 || (kz == -1 && errno == EINTR) ) continue;
+	if( kz < 0) continue;
+	if( FD_ISSET(s_in.cl_sock, &rd_fd) )
 	{
 	    r_len = read(s_in.cl_sock,buf,buf_len*1000);
-	    Mess->put(s_in.s_in->nodePath().c_str(),TMess::Info,mod->I18N("Socket receive of message <%d> from <%s>!"), r_len, s_in.sender.c_str() );
-	    if(r_len > 0) 
-	    {
-		req.assign(buf,r_len);
+	    //mess_debug(s_in.s_in->nodePath().c_str(),_("Socket receive of message <%d> from <%s>!"), r_len, s_in.sender.c_str() );
+    	    if(r_len <= 0) break;
+    	    req.assign(buf,r_len);
 
-		PutMess(s_in.cl_sock,req,answ,s_in.sender,prot_in);
-		if( answ.size() && prot_in.freeStat()  ) 
-		{
-		    Mess->put(s_in.s_in->nodePath().c_str(),TMess::Info,mod->I18N("Socket reply of message <%d> to <%s>!"), answ.size(), s_in.sender.c_str() );
-		    r_len = write(s_in.cl_sock,answ.c_str(),answ.size());   
-		}		
+	    PutMess(s_in.cl_sock,req,answ,s_in.sender,prot_in);
+	    if( prot_in.freeStat() && answ.size() )
+	    {
+    		//mess_debug(s_in.s_in->nodePath().c_str(),_("Socket reply of message <%d> to <%s>!"), answ.size(), s_in.sender.c_str() );	    
+		r_len = write(s_in.cl_sock,answ.c_str(),answ.size());
 	    }
-	    else break;
 	}
-	while( !prot_in.freeStat() );
-    }
+    }while( !endrun_cl && (mode || !prot_in.freeStat()) );
 }
 
 void TSocketIn::PutMess( int sock, string &request, string &answer, string sender, AutoHD<TProtocolIn> &prot_in )
 {
+    AutoHD<TProtocol> proto;
+    string n_pr = id()+TSYS::int2str(sock);
     try
     {
-	AutoHD<TProtocol> proto = SYS->protocol().at().modAt(protocol());
-	string n_pr = id()+TSYS::int2str(sock);
-    
+	proto = SYS->protocol().at().modAt(protocol());    
         if( prot_in.freeStat() ) 
 	{
 	    if( !proto.at().openStat(n_pr) ) proto.at().open( n_pr );
@@ -511,33 +472,33 @@ void TSocketIn::PutMess( int sock, string &request, string &answer, string sende
 	}
 	if( prot_in.at().mess(request,answer,sender) ) return;
 	prot_in.free();
-	if( proto.at().openStat(n_pr) ) proto.at().close( n_pr );
-    
-	prot_in.free();
-	if( proto.at().openStat(n_pr) ) proto.at().close( n_pr );
+	if( proto.at().openStat(n_pr) ) proto.at().close(n_pr);    
     }catch(TError err)
     { 
-	Mess->put(nodePath().c_str(),TMess::Error,"%s",err.mess.c_str() );
-	Mess->put(nodePath().c_str(),TMess::Error,mod->I18N("Put message error."));
+	prot_in.free();
+	if( !proto.freeStat() && proto.at().openStat(n_pr) ) proto.at().close( n_pr );
+	
+	mess_err(nodePath().c_str(),"%s",err.mess.c_str() );
+	mess_err(nodePath().c_str(),_("Error request into protocol."));
     }
 }
 
-void TSocketIn::RegClient(pid_t pid, int i_sock)
+void TSocketIn::RegClient(pthread_t thrid, int i_sock)
 {
     ResAlloc res(sock_res,true);
-    //find already registry
+    //- Find already registry -
     for( unsigned i_id = 0; i_id < cl_id.size(); i_id++)
-	if( cl_id[i_id].cl_pid == pid ) return;
-    SSockCl scl = { pid, i_sock };
+	if( cl_id[i_id].cl_id == thrid ) return;
+    SSockCl scl = { thrid, i_sock };
     cl_id.push_back(scl);
     cl_free = false;
 }
 
-void TSocketIn::UnregClient(pid_t pid)
+void TSocketIn::UnregClient(pthread_t thrid)
 {
     ResAlloc res(sock_res,true);
     for( unsigned i_id = 0; i_id < cl_id.size(); i_id++ ) 
-	if( cl_id[i_id].cl_pid == pid ) 
+	if( cl_id[i_id].cl_id == thrid ) 
 	{
 	    shutdown(cl_id[i_id].cl_sock,SHUT_RDWR);
 	    close(cl_id[i_id].cl_sock);
@@ -547,41 +508,47 @@ void TSocketIn::UnregClient(pid_t pid)
 	}
 }
 
-void TSocketIn::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
+void TSocketIn::cntrCmdProc( XMLNode *opt )
 {
-    if( cmd==TCntrNode::Info )
-    {	
-	TTransportIn::cntrCmd_( a_path, opt, cmd );
-
-	ctrMkNode("area",opt,-1,a_path.c_str(),"/bs",mod->I18N(MOD_NAME));	
-	ctrMkNode("fld",opt,-1,a_path.c_str(),"/bs/q_ln",mod->I18N("Queue length for TCP and UNIX sockets"),0660,0,0,1,"tp","dec");
-	ctrMkNode("fld",opt,-1,a_path.c_str(),"/bs/cl_n",mod->I18N("Maximum number opened client TCP and UNIX sockets"),0660,0,0,1,"tp","dec");
-	ctrMkNode("fld",opt,-1,a_path.c_str(),"/bs/bf_ln",mod->I18N("Input buffer length (kbyte)"),0660,0,0,1,"tp","dec");
-    }
-    else if( cmd==TCntrNode::Get )
+    //- Get page info -
+    if( opt->name() == "info" )
     {
-	if( a_path == "/bs/q_ln" )	ctrSetI( opt, max_queue );
-	else if( a_path == "/bs/cl_n" )	ctrSetI( opt, max_fork );
-	else if( a_path == "/bs/bf_ln" )ctrSetI( opt, buf_len );
-    	else TTransportIn::cntrCmd_( a_path, opt, cmd );
+	TTransportIn::cntrCmdProc(opt);
+	if(ctrMkNode("area",opt,-1,"/bs",_(MOD_NAME)))
+	{
+	    ctrMkNode("fld",opt,-1,"/bs/q_ln",_("Queue length for TCP and UNIX sockets"),0660,"root","root",1,"tp","dec");
+	    ctrMkNode("fld",opt,-1,"/bs/cl_n",_("Maximum number opened client TCP and UNIX sockets"),0660,"root","root",1,"tp","dec");
+	    ctrMkNode("fld",opt,-1,"/bs/bf_ln",_("Input buffer length (kbyte)"),0660,"root","root",1,"tp","dec");
+	}
+	return;
     }
-    else if( cmd==TCntrNode::Set )
+    //- Process command to page -
+    string a_path = opt->attr("path");
+    if( a_path == "/bs/q_ln" )
     {
-	if( a_path == "/bs/q_ln" )        max_queue = ctrGetI( opt );
-	else if( a_path == "/bs/cl_n" )   max_fork  = ctrGetI( opt );
-	else if( a_path == "/bs/bf_ln" )  buf_len   = ctrGetI( opt );
-	else TTransportIn::cntrCmd_( a_path, opt, cmd );
+	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )	opt->setText(TSYS::int2str(max_queue));
+	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	max_queue = atoi(opt->text().c_str());
     }
-}    
-
-//==============================================================================
-//== TSocketOut ================================================================
-//==============================================================================
-
+    else if( a_path == "/bs/cl_n" )
+    {
+	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) )	opt->setText(TSYS::int2str(max_fork));
+	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	max_fork = atoi(opt->text().c_str());
+    }
+    else if( a_path == "/bs/bf_ln" )
+    {
+	if( ctrChkNode(opt,"get",0660,"root","root",SEQ_RD) ) 	opt->setText(TSYS::int2str(buf_len));
+	if( ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	buf_len = atoi(opt->text().c_str());
+    }
+    else TTransportIn::cntrCmdProc(opt);
+}
+	    
+//************************************************
+//* TSocketOut                                   *
+//************************************************
 TSocketOut::TSocketOut(string name, const string &idb, TElem *el) : 
     TTransportOut(name,idb,el), sock_fd(-1)
 {
-    
+    setAddr("TCP:localhost:10002");
 }
 
 TSocketOut::~TSocketOut()
@@ -598,7 +565,7 @@ void TSocketOut::start()
     if( s_type == S_NM_TCP ) 		type = SOCK_TCP;
     else if( s_type == S_NM_UDP ) 	type = SOCK_UDP;
     else if( s_type == S_NM_UNIX )	type = SOCK_UNIX;
-    else throw TError(nodePath().c_str(),mod->I18N("Type socket <%s> error!"),s_type.c_str());
+    else throw TError(nodePath().c_str(),_("Type socket <%s> error!"),s_type.c_str());
 
     if( type == SOCK_TCP || type == SOCK_UDP )
     {
@@ -607,37 +574,37 @@ void TSocketOut::start()
         
 	string host = TSYS::strSepParse(addr(),1,':');
         string port = TSYS::strSepParse(addr(),2,':');
-	if( !host.size() )
+	if( host.size() )
 	{
    	    struct hostent *loc_host_nm = gethostbyname(host.c_str());
 	    if(loc_host_nm == NULL || loc_host_nm->h_length == 0)
-		throw TError(nodePath().c_str(),mod->I18N("Socket name <%s> error!"),host.c_str());
+		throw TError(nodePath().c_str(),_("Socket name <%s> error!"),host.c_str());
 	    name_in.sin_addr.s_addr = *( (int *) (loc_host_nm->h_addr_list[0]) );
 	}
-	else name_in.sin_addr.s_addr = INADDR_ANY;  
-	//Get system port for "oscada" /etc/services
+	else name_in.sin_addr.s_addr = INADDR_ANY;
+	//- Get system port for "oscada" /etc/services -
 	struct servent *sptr = getservbyname(port.c_str(),(type == SOCK_TCP)?"tcp":"udp");
 	if( sptr != NULL )                       name_in.sin_port = sptr->s_port;
 	else if( htons(atol(port.c_str())) > 0 ) name_in.sin_port = htons( atol(port.c_str()) );
 	else name_in.sin_port = 10001;
 	
-	//Create socket
+	//- Create socket -
 	if( type == SOCK_TCP )
 	{
 	    if( (sock_fd = socket(PF_INET,SOCK_STREAM,0) )== -1 )
-        	throw TError(nodePath().c_str(),mod->I18N("Error create TCP socket: %s!"),strerror(errno));
+        	throw TError(nodePath().c_str(),_("Error create TCP socket: %s!"),strerror(errno));
 	}	
 	else if( type == SOCK_UDP )
 	{
 	    if( (sock_fd = socket(PF_INET,SOCK_DGRAM,0) )== -1 )
-        	throw TError(nodePath().c_str(),mod->I18N("Error create UDP socket: %s!"),strerror(errno));
+        	throw TError(nodePath().c_str(),_("Error create UDP socket: %s!"),strerror(errno));
 	}
-	//Connect to socket		
+	//- Connect to socket -
 	if( ::connect(sock_fd, (sockaddr *)&name_in, sizeof(name_in)) == -1 )
 	{
 	    close(sock_fd);
 	    sock_fd = -1;
-            throw TError(nodePath().c_str(),mod->I18N("Connect to Internet socket error: %s!"),strerror(errno));
+            throw TError(nodePath().c_str(),_("Connect to Internet socket error: %s!"),strerror(errno));
 	}
     }
     else if( type == SOCK_UNIX )
@@ -648,14 +615,14 @@ void TSocketOut::start()
 	name_un.sun_family = AF_UNIX;
 	strncpy( name_un.sun_path,path.c_str(),sizeof(name_un.sun_path) );
 	
-	//Create socket	
+	//- Create socket -
 	if( (sock_fd = socket(PF_UNIX,SOCK_STREAM,0) )== -1)
-            throw TError(nodePath().c_str(),mod->I18N("Error create UNIX socket: %s!"),strerror(errno));
+            throw TError(nodePath().c_str(),_("Error create UNIX socket: %s!"),strerror(errno));
 	if( ::connect(sock_fd, (sockaddr *)&name_un, sizeof(name_un)) == -1 )
 	{
 	    close(sock_fd);
 	    sock_fd = -1;
-            throw TError(nodePath().c_str(),mod->I18N("Connect to UNIX error: %s!"),strerror(errno));
+            throw TError(nodePath().c_str(),_("Connect to UNIX error: %s!"),strerror(errno));
 	}	    
     }    
     run_st = true;
@@ -676,20 +643,17 @@ void TSocketOut::stop()
 int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, int time )
 {
     int kz;
-    if( !run_st ) throw TError(nodePath().c_str(),mod->I18N("Transport no started!"));    
+    if( !run_st ) throw TError(nodePath().c_str(),_("Transport no started!"));    
     
-    //Write request
+    //- Write request -
     if( obuf != NULL && len_ob > 0)
-    {	
-	kz = write(sock_fd,obuf,len_ob);
-	if( kz < 0)
+	while( (kz = write(sock_fd,obuf,len_ob)) <= 0)
         {
             run_st = false;
-            throw TError(nodePath().c_str(),mod->I18N("Socket error!"));
+	    start();
 	}
-    }
 
-    //Read reply
+    //- Read reply -
     int i_b = 0;
     if( ibuf != NULL && len_ib > 0 && time > 0 )
     {
@@ -706,7 +670,7 @@ int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, in
 	else if( kz < 0)
 	{ 
 	    run_st = false;	    
-	    throw TError(nodePath().c_str(),mod->I18N("Socket error!"));
+	    throw TError(nodePath().c_str(),_("Socket error!"));
 	    
 	}
 	else if( FD_ISSET(sock_fd, &rd_fd) )
@@ -715,13 +679,13 @@ int TSocketOut::messIO( const char *obuf, int len_ob, char *ibuf, int len_ib, in
 	    if(i_b < 0)	
 	    {
 		run_st = false;		
-		throw TError(nodePath().c_str(),mod->I18N("Read reply error: %s"),strerror(errno));
+		throw TError(nodePath().c_str(),_("Read reply error: %s"),strerror(errno));
 	    }
 	}
 	else
 	{ 
 	    run_st = false;
-	    throw TError(nodePath().c_str(),mod->I18N("Timeouted!"));
+	    throw TError(nodePath().c_str(),_("Timeouted!"));
 	}
     }
 
