@@ -1,13 +1,12 @@
 
 //OpenSCADA system file: tarchives.h
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -38,63 +37,70 @@ using std::vector;
 
 #define BUF_ARCH_NM "<bufer>"
 
-//================================================================
-//===================== Message archivator =======================
-//================================================================
+//************************************************
+//* Message archivator                           *
+//************************************************
 
-//================ TMArchivator ==================================
+//************************************************
+//* TMArchivator                                 *
+//************************************************
 class TTipArchivator;
 
 class TMArchivator : public TCntrNode, public TConfig
 {
     public:
-	//Methods
-	TMArchivator(const string &iid, const string &idb, TElem *cf_el );
+	//Public methods
+	TMArchivator(const string &id, const string &db, TElem *cf_el );
 
-        const string &id()	{ return m_id; }
-	string workId();
-        string name();
-        string dscr()		{ return m_dscr; }
-	bool toStart() 		{ return m_start; }
-	bool startStat()	{ return run_st; }
+        const string &id( )	{ return m_id; }
+	string workId( );
+        string name( );
+        string dscr( )		{ return m_dscr; }
+	bool toStart( ) 	{ return m_start; }
+	bool startStat( )	{ return run_st; }
+        string &addr( )		{ return m_addr; }
+        int    &level( )	{ return m_level; }
+        void   categ( vector<string> &list );	
 
-	void name( const string &vl )  	{ m_name = vl; }
-        void dscr( const string &vl )  	{ m_dscr = vl; }
-	void toStart( bool vl )		{ m_start = vl; }
+	string DB( )            { return m_db; }
+        string tbl( );
+        string fullDB( )        { return DB()+'.'+tbl(); }
+
+	void setName( const string &vl )  	{ m_name = vl; }
+        void setDscr( const string &vl )  	{ m_dscr = vl; }
+	void setToStart( bool vl )		{ m_start = vl; }
+	void setAddr( const string &vl )	{ m_addr = vl; }
 
         virtual void load( );
         virtual void save( );
         virtual void start( )	{ };
         virtual void stop( )	{ };
 
-        string &addr()	{ return m_addr; }
-        int    &level()	{ return m_level; }
-        void   categ( vector<string> &list );
-
+        virtual time_t begin( )	{ return 0; }
+        virtual time_t end( ) 	{ return 0; }
 	virtual void put( vector<TMess::SRec> &mess ){ };
         virtual void get( time_t b_tm, time_t e_tm, vector<TMess::SRec> &mess, const string &category = "", char level = 0 ) { };
 
-	string BD();
-
 	TTipArchivator &owner()	{ return *(TTipArchivator *)nodePrev(); }
 
     protected:
-	//Methods
-	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
-	void postEnable( );
-	void postDisable(int flag);     //Delete all DB if flag 1
+	//Protected methods
+	void cntrCmdProc( XMLNode *opt );       //Control interface command process
+	void postEnable( int flag );
+	void preDisable( int flag );
+	void postDisable( int flag );     //Delete all DB if flag 1
 
 	//- Check messages criteries -
 	bool chkMessOK( const string &icateg, TMess::Type ilvl );
 
-	//Attributes
+	//Protected atributes
 	bool           run_st;
 
     private:
-	//Methods
+	//Private methods
         string nodeName()       { return m_id; }
 
-	//Attributes
+	//Private attributes
 	string	&m_id,		//Mess arch id
 		&m_name,	//Mess arch name
 		&m_dscr,	//Mess arch description
@@ -102,24 +108,22 @@ class TMArchivator : public TCntrNode, public TConfig
 		&m_cat_o;	//Mess arch cetegory
 	bool	&m_start;	//Mess arch starting flag
 	int	&m_level;	//Mess arch level
-	string  m_bd;
-
-	//- Request mess params -
-	time_t	m_beg, m_end;
-	string	m_cat;
-	int	m_lvl;
+	string  m_db;
 };
 
-//================================================================
-//=================== Archive subsystem ==========================
-//================================================================
+//************************************************
+//* Archive subsystem                            *
+//************************************************
 
-//=================== TTipArchivator =============================
+//************************************************
+//* TTipArchivator                               *
+//************************************************
 class TArchiveS;
 
 class TTipArchivator: public TModule
 {
     public:
+	//Public methods
     	TTipArchivator( );
 	virtual ~TTipArchivator();
 
@@ -128,48 +132,48 @@ class TTipArchivator: public TModule
         bool messPresent( const string &iid )	{ return chldPresent(m_mess,iid); }
 	void messAdd( const string &iid, const string &idb = "*.*" );
 	void messDel( const string &iid, bool full = false )	{ chldDel(m_mess,iid,-1,full); }
-	AutoHD<TMArchivator> messAt( const string &iid )
- 	{ return chldAt(m_mess,iid); }
+	AutoHD<TMArchivator> messAt( const string &iid ) 	{ return chldAt(m_mess,iid); }
 
 	//- Values -
 	void valList( vector<string> &list )	{ chldList(m_val,list); }
         bool valPresent( const string &iid )	{ return chldPresent(m_val,iid); }
 	void valAdd( const string &iid, const string &idb = "*.*" );
         void valDel( const string &iid, bool full = false )	{ chldDel(m_val,iid,-1,full); }
-	AutoHD<TVArchivator> valAt( const string &iid )
- 	{ return chldAt(m_val,iid); }
+	AutoHD<TVArchivator> valAt( const string &iid ) 	{ return chldAt(m_val,iid); }
 
 	TArchiveS &owner();
 
     protected:
-	//Methods
-	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
+	//Protected methods
+	void cntrCmdProc( XMLNode *opt );       //Control interface command process
 	
-	virtual TMArchivator *AMess(const string &iid, const string &idb )
+	virtual TMArchivator *AMess(const string &id, const string &db )
 	{ throw TError(nodePath().c_str(),"Message arhiv no support!"); }
-	virtual TVArchivator *AVal(const string &iid, const string &idb )
+	virtual TVArchivator *AVal(const string &id, const string &db )
 	{ throw TError(nodePath().c_str(),"Value arhiv no support!"); }	
 
     private:
-	//Attributes
+	//Private attributes
 	int	m_mess, m_val;
 };
 
-//===================== TArchiveS ================================
+//************************************************
+//* TArchiveS                                    *
+//************************************************
 class TArchiveS : public TSubSYS
 {
     public:
-	//Methods
+	//Public methods
 	TArchiveS( );
 	~TArchiveS(  );
 
-	int subVer( )	{ return VER_ARH; }
+	int subVer( )		{ return VER_ARH; }
 	
-	int messPeriod(){ return m_mess_per; }
-	int valPeriod()	{ return m_val_per; }
+	int messPeriod( )	{ return m_mess_per; }
+	int valPeriod( )	{ return m_val_per; }
 	
-	void messPeriod( int ivl );
-	void valPeriod( int ivl )	{ m_val_per = ivl; }
+	void setMessPeriod( int ivl );
+	void setValPeriod( int ivl )	{ m_val_per = ivl; }
 	
 	void subLoad( );
 	void subSave( );
@@ -177,45 +181,45 @@ class TArchiveS : public TSubSYS
 	void subStop( );
 
 	//- Value archives functions -
-	void valList( vector<string> &list )	{ chldList(m_aval,list); }
-        bool valPresent( const string &iid )  	{ return chldPresent(m_aval,iid); }
+	void valList( vector<string> &list )			{ chldList(m_aval,list); }
+        bool valPresent( const string &iid )  			{ return chldPresent(m_aval,iid); }
 	void valAdd( const string &iid, const string &idb = "*.*" );
 	void valDel( const string &iid, bool db = false )	{ chldDel(m_aval,iid,-1,db); }
-	AutoHD<TVArchive> valAt( const string &iid )
-        { return chldAt(m_aval,iid); }
+	AutoHD<TVArchive> valAt( const string &iid )		{ return chldAt(m_aval,iid); }
 
 	void setActValArch( const string &id, bool val );
 
 	//- Archivators -
-	AutoHD<TTipArchivator> at( const string &name )	{ return modAt(name); }
+	AutoHD<TTipArchivator> at( const string &name )		{ return modAt(name); }
 
 	//- Message archive function -
  	void messPut( time_t tm, const string &categ, TMess::Type level, const string &mess );	
 	void messPut( const vector<TMess::SRec> &recs );
-        void messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const string &category = "", TMess::Type level = TMess::Debug, const string &arch = "" );
+        void messGet( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const string &category = "", 
+		TMess::Type level = TMess::Debug, const string &arch = "" );
+	time_t messBeg( const string &arch = "" );
+	time_t messEnd( const string &arch = "" );
 
-	TElem &messE()	{ return el_mess; }
-	TElem &valE() 	{ return el_val; }
-	TElem &aValE()	{ return el_aval; }
+	TElem &messE( )		{ return el_mess; }
+	TElem &valE( ) 		{ return el_val; }
+	TElem &aValE( )		{ return el_aval; }
 	
-	//Attributes
+	//Public attributes
         static int max_req_mess;
-        static int max_req_vals;
 
     private:
-	//Methods
+	//Private methods
 	string optDescr(  );
 
 	static void ArhMessTask( union sigval obj );
 	static void *ArhValTask( void *param );
 
-	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
+	void cntrCmdProc( XMLNode *opt );       //Control interface command process
 
 	int messBufLen( )	{ return m_buf.size(); }
 	void messBufLen(int len);
 
-    private:
-	//Attributes
+	//Private attributes
 	TElem  		el_mess, 	//Message archivator's DB elements
 			el_val, 	//Value archivator's DB elements
 			el_aval;	//Value archives DB elements
@@ -226,23 +230,20 @@ class TArchiveS : public TSubSYS
 	timer_t	tmIdMess;   		//Messages timer
 	bool   	prc_st_mess;		//Process messages flag
 	//-- Messages buffer --
-	int    	m_res;			//Mess access resource
+	Res    	m_res;			//Mess access resource
 	unsigned head_buf, 		//Head of messages buffer
 		head_lstread;    	//Last read and archived head of messages buffer
 	vector<TMess::SRec> m_buf; 	//Messages buffer
- 	//-- Request mess params --
-	time_t	m_beg, m_end;
-	string	m_cat;
-	int	m_lvl;
-	string	m_archtor;
 	
 	//- Value archiving -
+ 	Res	v_res;			//Value access resource
 	int     m_val_per;             	//Value arhiving period
 	int	m_val_prior;		//Value archive task priority
 	pthread_t m_val_pthr;		//Value get realtime pthread	
 	bool	prc_st_val;		//Process value flag
 	bool    endrun_req_val;		//Endrun value request
-	int	m_aval, v_res;
+	int	m_aval;
+
 	vector< AutoHD<TVArchive> > act_up_src;
 };
 
