@@ -1,13 +1,12 @@
 
 //OpenSCADA system file: tcontroller.h
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -31,12 +30,15 @@
 
 using std::string;
 
+//*************************************************
+//* TController                                   *
+//*************************************************
 class TTipDAQ;
 
 class TController : public TCntrNode, public TConfig
 {
     public:
-	//Methods
+	//Public methods
      	TController( const string &name_c, const string &daq_db, TElem *cfgelem );
 	virtual ~TController(  );
 	
@@ -44,8 +46,12 @@ class TController : public TCntrNode, public TConfig
 	string name();
 	string descr()		{ return m_descr; }
 	
-	void name( const string &nm ) 	{ m_name = nm; }
-	void descr( const string &dscr ){ m_descr = dscr; }	
+        string DB( )            { return m_db; }
+        string tbl( );
+        string fullDB( )        { return DB()+'.'+tbl(); }
+	
+	void setName( const string &nm ) 	{ m_name = nm; }
+	void setDescr( const string &dscr )	{ m_descr = dscr; }	
 	
     	bool toEnable()		{ return m_aen; }
     	bool toStart() 		{ return m_astart; }
@@ -54,8 +60,8 @@ class TController : public TCntrNode, public TConfig
 
 	virtual void load( );
 	virtual void save( );
-	virtual void start( );
-	virtual void stop( );
+	void start( );
+	void stop( );
 	void enable( );
 	void disable( );
 
@@ -67,44 +73,43 @@ class TController : public TCntrNode, public TConfig
         AutoHD<TParamContr> at( const string &name, const string &who = "th_contr" )
 	{ return chldAt(m_prm,name); }
 	
-	string genBD()	{ return m_bd; }
-	string BD();	
-		
-	TTipDAQ &owner(){ return *(TTipDAQ *)nodePrev(); }
+	TTipDAQ &owner()	{ return *(TTipDAQ *)nodePrev(); }
 	
     protected:    
-	//Attributes
+	//Protected attributes
 	bool    en_st;    
 	bool    run_st;
 	
 	//Methods
-	virtual void enable_(  ) { }
-        virtual void disable_(  ){ }
+	//- User methods -
+	virtual void enable_(  ) 	{ }
+        virtual void disable_(  )	{ }
+	virtual void start_( )	 	{ }
+        virtual void stop_( )		{ }
     
 	virtual TParamContr *ParamAttach( const string &name, int type );    
     
-	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
+	void cntrCmdProc( XMLNode *opt );       //Control interface command process
 	
 	void preDisable(int flag);	//Disable if delete
 	void postDisable(int flag);     //Delete all DB if flag 1
 	
     private:
-	//Methods
+	//Private methods
 	string nodeName()       { return m_id; }
     
 	void LoadParmCfg(  );
 	void SaveParmCfg(  );
 	void FreeParmCfg(  );
 	
-	//Attributes
+	//Private attributes
 	string  &m_id;
         string  &m_name;
         string  &m_descr;
         bool    &m_aen;
         bool    &m_astart;
-        int     m_add_type;    //Add parameter type
 	
-	string	m_bd;
+	string	m_db;
 	int	m_prm;
 };
 
