@@ -1,13 +1,12 @@
 
 //OpenSCADA system file: terror.h
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -26,17 +25,21 @@
 #include <string>
 #include <exception>
 
-
 using std::string;
 using std::exception;
 
+//*************************************************
+//* TError                                        *
+//*************************************************
 class TError
 {
     public:
 	//Methods
-	TError( const char *icat, const char *fmt, ... );
+	TError( const char *cat, const char *fmt, ... );
+	TError( int cod, const char *cat, const char *fmt, ... );
 	
 	//Attributes
+	int	cod;
 	string  cat;
 	string  mess;
 };
