@@ -1,13 +1,12 @@
 
 //OpenSCADA system file: tmess.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -34,12 +33,13 @@
 #include <errno.h>
 
 #include "../config.h"
-//#include "terror.h"
 #include "tsys.h"
 #include "resalloc.h"
-//#include "tarchives.h"
 #include "tmess.h"
 
+//*************************************************
+//* TMess                                         *
+//*************************************************
 TMess::TMess(  ) : IOCharSet("UTF8"), m_mess_level(0), log_dir(0x2)
 {
     openlog(PACKAGE,0,LOG_USER);
@@ -78,15 +78,15 @@ void TMess::put( const char *categ, Type level, const char *fmt,  ... )
 	case Warning:	level_sys = LOG_WARNING;break;
 	case Error:	level_sys = LOG_ERR;	break;
 	case Crit:	level_sys = LOG_CRIT;	break;
-	case Allert:	level_sys = LOG_ALERT;	break;
+	case Alert:	level_sys = LOG_ALERT;	break;
 	case Emerg:	level_sys = LOG_EMERG;	break;
 	default: 	level_sys = LOG_DEBUG;
     }
     string s_mess = TSYS::int2str(level) + "|" + categ + " | " + mess;
-    if(log_dir&1) syslog(level_sys,s_mess.c_str());
+    if(log_dir&1) syslog(level_sys,TSYS::strEncode(s_mess,TSYS::FormatPrint).c_str());
     if(log_dir&2) fprintf(stdout,"%s \n",s_mess.c_str());
     if(log_dir&4) fprintf(stderr,"%s \n",s_mess.c_str());
-    if(log_dir&8) SYS->archive().at().messPut(time(NULL),categ,level,mess);
+    if((log_dir&8) && SYS->present("Archive") )	SYS->archive().at().messPut(time(NULL),categ,level,mess);
 }
 
 void TMess::get( time_t b_tm, time_t e_tm, vector<TMess::SRec> & recs, const string &category, Type level )
@@ -102,7 +102,7 @@ string TMess::lang( )
     else return("C");
 }
 
-void TMess::lang( const string &lng )
+void TMess::setLang( const string &lng )
 {
     if( getenv("LANGUAGE") ) setenv("LANGUAGE", lng.c_str(), 1);
     else setenv("LC_MESSAGES", lng.c_str(), 1);
@@ -114,7 +114,7 @@ void TMess::lang( const string &lng )
 
 string TMess::codeConv( const string &fromCH, const string &toCH, const string &mess)
 {
-    //Make convert to blocks 100 bytes !!!    
+    //- Make convert to blocks 100 bytes !!! -
     string buf = ""; 
     char   *ibuf, outbuf[100], *obuf;
     size_t ilen, olen;
@@ -123,7 +123,7 @@ string TMess::codeConv( const string &fromCH, const string &toCH, const string &
     hd = iconv_open(toCH.c_str(), fromCH.c_str());
     if( hd == (iconv_t)(-1) )
     {
-	put("IConv",TMess::Crit,I18N("Error iconv open: %s"),strerror(errno));
+	mess_crit("IConv",_("Error iconv open: %s"),strerror(errno));
     	return mess;
     }
         
@@ -137,7 +137,8 @@ string TMess::codeConv( const string &fromCH, const string &toCH, const string &
 	size_t rez = iconv(hd,&ibuf,&ilen,&obuf,&olen);
 	if( rez == (size_t)(-1) && errno != E2BIG )
 	{
-	    put("IConv",TMess::Crit,I18N("Error input sequence convert: %s"),strerror(errno));
+	    mess_crit("IConv",_("Error input sequence convert: %s"),strerror(errno));
+	    mess_debug("IConv",_("Error converting from %s to %s for message: %s"),fromCH.c_str(),toCH.c_str(),mess.c_str());
 	    buf = mess;
 	    break;
 	}
@@ -188,7 +189,7 @@ bool TMess::chkPattern( const string &val, const string &patt )
 
 void TMess::load()
 {
-    //======================= Load params from command line =========================
+    //- Load params from command line -
     int i,next_opt;
     char *short_opt="h";
     struct option long_opt[] =
@@ -206,16 +207,16 @@ void TMess::load()
 	switch(next_opt)
 	{
 	    case 'h': return;
-	    case 'd': i = atoi(optarg); if(i>=0&&i<=7) messLevel(i); break;
-	    case 'l': logDirect(atoi(optarg)); break;
+	    case 'd': i = atoi(optarg); if(i>=0&&i<=7) setMessLevel(i); break;
+	    case 'l': setLogDirect(atoi(optarg)); break;
 	    case -1 : break;
 	}
     } while(next_opt != -1);
     
-    //======================= Load params config file =========================
-    i = atoi(TBDS::genDBGet(SYS->nodePath()+"MessLev",TSYS::int2str(messLevel()),SYS->sysOptCfg()).c_str());
-    if( i >= 0 && i <= 7 ) messLevel(i);
-    logDirect(atoi(TBDS::genDBGet(SYS->nodePath()+"LogTarget",TSYS::int2str(logDirect()),SYS->sysOptCfg()).c_str()));
+    //- Load params config file -
+    i = atoi(TBDS::genDBGet(SYS->nodePath()+"MessLev",TSYS::int2str(messLevel()),"root",SYS->sysOptCfg()).c_str());
+    if( i >= 0 && i <= 7 ) setMessLevel(i);
+    setLogDirect(atoi(TBDS::genDBGet(SYS->nodePath()+"LogTarget",TSYS::int2str(logDirect()),"root",SYS->sysOptCfg()).c_str()));
 }
 
 void TMess::save()
@@ -223,5 +224,3 @@ void TMess::save()
     TBDS::genDBSet(SYS->nodePath()+"MessLev",TSYS::int2str(messLevel()));
     TBDS::genDBSet(SYS->nodePath()+"LogTarget",TSYS::int2str(logDirect()));
 }
-
-
