@@ -1,13 +1,12 @@
 
 //OpenSCADA system file: tmodschedul.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -32,15 +31,15 @@
 #include <string>
 
 #include "tsys.h"
-#include "resalloc.h"
 #include "tmodschedul.h"
 
+//*************************************************
+//* TModSchedul                                   *
+//*************************************************
 TModSchedul::TModSchedul( ) : 
     TSubSYS("ModSched","Modules sheduler",false), prc_st(false), m_mod_path("./"), m_per(10)
 {
-    hd_res = ResAlloc::resCreate();
-
-    //Create calc timer
+    //- Create calc timer -
     struct sigevent sigev;
     sigev.sigev_notify = SIGEV_THREAD;
     sigev.sigev_value.sival_ptr = this;
@@ -52,33 +51,32 @@ TModSchedul::TModSchedul( ) :
 TModSchedul::~TModSchedul(  )
 {
     timer_delete(tmId);
-    ResAlloc::resDelete(hd_res);
 }
 
 void TModSchedul::preDisable(int flag)
 {
-    subStop();
-	
-    //Detach all share libs
+    //- Detach all share libs -
     ResAlloc res(hd_res,true);
     for( unsigned i_sh = 0; i_sh < SchHD.size(); i_sh++ )
         if( SchHD[i_sh]->hd )
         {
             while( SchHD[i_sh]->use.size() )
             {
-                owner().at(SchHD[i_sh]->use[0].mod_sub).at().modDel( SchHD[i_sh]->use[0].n_mod );
+                owner().at(TSYS::strSepParse(SchHD[i_sh]->use[0],0,'.')).at().
+			modDel(TSYS::strSepParse(SchHD[i_sh]->use[0],1,'.'));
                 SchHD[i_sh]->use.erase(SchHD[i_sh]->use.begin());
     	    }
             dlclose(SchHD[i_sh]->hd);
-    	    SchHD[i_sh]->hd = NULL;
+	    delete SchHD[i_sh];
 	}
+    SchHD.clear();	
     res.release();
 }
 
 string TModSchedul::optDescr( )
 {
     char buf[STR_BUF_LEN];
-    snprintf(buf,sizeof(buf),Mess->I18N(
+    snprintf(buf,sizeof(buf),_(
 	"=================== Subsystem \"Module sheduler\" options =================\n"
     	"    --ModPath=<path>   Modules <path> (/var/os/modules/).\n"
 	"------------ Parameters of section <%s> in config file -----------\n"
@@ -92,7 +90,11 @@ string TModSchedul::optDescr( )
 
 void TModSchedul::subStart(  )
 { 
-    //Start interval timer for periodic thread creating
+#if OSC_DEBUG
+    mess_debug(nodePath().c_str(),_("Start subsystem."));
+#endif
+
+    //- Start interval timer for periodic thread creating -
     struct itimerspec itval;
     itval.it_interval.tv_sec = itval.it_value.tv_sec = m_per;
     itval.it_interval.tv_nsec = itval.it_value.tv_nsec = 0;
@@ -101,13 +103,21 @@ void TModSchedul::subStart(  )
 
 void TModSchedul::subStop(  )
 {
-    //Stop interval timer for periodic thread creating
+#if OSC_DEBUG
+    mess_debug(nodePath().c_str(),_("Stop subsystem."));
+#endif
+
+    //- Stop interval timer for periodic thread creating -
     struct itimerspec itval;
     itval.it_interval.tv_sec = itval.it_interval.tv_nsec =
     	itval.it_value.tv_sec = itval.it_value.tv_nsec = 0;
     timer_settime(tmId, 0, &itval, NULL);
     if( TSYS::eventWait( prc_st, false, nodePath()+"stop",20) )
-	throw TError(nodePath().c_str(),Mess->I18N("Module scheduler thread no stoped!"));
+	throw TError(nodePath().c_str(),_("Module scheduler thread no stoped!"));
+
+#if OSC_DEBUG
+    mess_debug(nodePath().c_str(),_("Stop subsystem. OK"));
+#endif    
 }
 
 void TModSchedul::chkPer( int per )
@@ -128,7 +138,7 @@ void TModSchedul::SchedTask(union sigval obj)
     try
     {
 	shed->libLoad(shed->m_mod_path,true);
-    } catch(TError err){ Mess->put(err.cat.c_str(),TMess::Error,"%s",err.mess.c_str()); }
+    } catch(TError err){ mess_err(err.cat.c_str(),"%s",err.mess.c_str()); }
 
     shed->prc_st = false;
 }
@@ -140,7 +150,7 @@ void TModSchedul::loadLibS(  )
 
 void TModSchedul::subLoad( )
 {
-    //===================== Load parameters from command line ================================
+    //- Load parameters from command line -
     int next_opt;
     char *short_opt="h";
     struct option long_opt[] =
@@ -162,18 +172,16 @@ void TModSchedul::subLoad( )
 	}
     } while(next_opt != -1);
     
-    //===================== Load parameters from command line ================================
+    //- Load parameters from command line -
     m_per = atoi(TBDS::genDBGet(nodePath()+"ChkPer",TSYS::int2str(m_per)).c_str());
     m_mod_path = TBDS::genDBGet(nodePath()+"ModPath",m_mod_path);
     
+
     string opt = TBDS::genDBGet(nodePath()+"ModAuto");
-    int el_cnt = 0;
-    m_am_list.clear();
-    while(TSYS::strSepParse(opt,el_cnt,':').size())
-    {
-        m_am_list.push_back(TSYS::strSepParse(opt,el_cnt,':'));
-        el_cnt++;
-    }
+    string ovl;    
+    m_am_list.clear();    
+    for( int el_off = 0; (ovl=TSYS::strSepParse(opt,0,';',&el_off)).size(); )
+        m_am_list.push_back(ovl);
 }
 
 void TModSchedul::subSave( )
@@ -182,7 +190,7 @@ void TModSchedul::subSave( )
     TBDS::genDBSet(nodePath()+"ModPath",m_mod_path);
     string m_auto;
     for(int i_a = 0; i_a < m_am_list.size(); i_a++ )
-	m_auto+=m_am_list[i_a]+":";
+	m_auto+=m_am_list[i_a]+";";
     TBDS::genDBSet(nodePath()+"ModAuto",m_auto);
 }
 
@@ -202,9 +210,6 @@ void TModSchedul::ScanDir( const string &Paths, vector<string> &files )
         Path=Paths.substr(ido,id-ido);
         if(Path.size() <= 0) continue;
 	
-	// Convert to absolutly path
-        //Path = SYS->fNameFix(Path);
-
         DIR *IdDir = opendir(Path.c_str());
         if(IdDir == NULL) continue;
 
@@ -230,10 +235,10 @@ bool TModSchedul::CheckFile( const string &iname )
     if( access(iname.c_str(),F_OK|R_OK) != 0 )  return false;
     NameMod=iname;
     
-    void *h_lib = dlopen(iname.c_str(),RTLD_GLOBAL|RTLD_LAZY);
+    void *h_lib = dlopen(iname.c_str(),RTLD_LAZY|RTLD_GLOBAL);
     if(h_lib == NULL)
     {
-        Mess->put(nodePath().c_str(),TMess::Warning,Mess->I18N("SO <%s> error: %s !"),iname.c_str(),dlerror());
+        mess_warning(nodePath().c_str(),_("SO <%s> error: %s !"),iname.c_str(),dlerror());
         return(false);
     }
     else dlclose(h_lib);        
@@ -265,7 +270,7 @@ int TModSchedul::libReg( const string &name )
     }
     else SchHD[i_sh]->m_tm = file_stat.st_mtime;
     
-    return(i_sh);    
+    return i_sh;    
 }
 
 void TModSchedul::libUnreg( const string &iname )
@@ -279,7 +284,7 @@ void TModSchedul::libUnreg( const string &iname )
 	    SchHD.erase(SchHD.begin()+i_sh);
 	    return;
 	}
-    throw TError(nodePath().c_str(),Mess->I18N("SO <%s> no present!"),iname.c_str());
+    throw TError(nodePath().c_str(),_("SO <%s> no present!"),iname.c_str());
 }
     
 void TModSchedul::libAtt( const string &iname, bool full )
@@ -289,28 +294,28 @@ void TModSchedul::libAtt( const string &iname, bool full )
        	if( SchHD[i_sh]->name == iname ) 
 	{
 	    if( SchHD[i_sh]->hd ) 
-		throw TError(nodePath().c_str(),Mess->I18N("SO <%s> already attached!"),iname.c_str());	    
+		throw TError(nodePath().c_str(),_("SO <%s> already attached!"),iname.c_str());	    
 	    
-	    void *h_lib = dlopen(iname.c_str(),RTLD_GLOBAL|RTLD_LAZY);	    
+	    void *h_lib = dlopen(iname.c_str(),RTLD_LAZY|RTLD_GLOBAL);
 	    if( !h_lib )
-		throw TError(nodePath().c_str(),Mess->I18N("SO <%s> error: %s !"),iname.c_str(),dlerror());	    
+		throw TError(nodePath().c_str(),_("SO <%s> error: %s !"),iname.c_str(),dlerror());	    
 	    
-	    //Connect to module function
+	    //- Connect to module function -
 	    TModule::SAt (*module)( int );
 	    module = (TModule::SAt (*)(int)) dlsym(h_lib,"module");
 	    if( dlerror() != NULL )
 	    {
 		dlclose(h_lib);
-		throw TError(nodePath().c_str(),Mess->I18N("SO <%s> error: %s !"),iname.c_str(),dlerror());
+		throw TError(nodePath().c_str(),_("SO <%s> error: %s !"),iname.c_str(),dlerror());
 	    }    
 	    
-	    //Connect to attach function	    
+	    //- Connect to attach function -
 	    TModule *(*attach)( const TModule::SAt &, const string & );
 	    attach = (TModule * (*)(const TModule::SAt &, const string &)) dlsym(h_lib,"attach");
 	    if( dlerror() != NULL )
 	    {
 		dlclose(h_lib);
-		throw TError(nodePath().c_str(),Mess->I18N("SO <%s> error: %s !"),iname.c_str(),dlerror());
+		throw TError(nodePath().c_str(),_("SO <%s> error: %s !"),iname.c_str(),dlerror());
 	    }    
 	    
 	    struct stat file_stat;
@@ -327,29 +332,28 @@ void TModSchedul::libAtt( const string &iname, bool full )
 		    if( owner().at(list[i_sub]).at().subModule() && 
 			AtMod.type == owner().at(list[i_sub]).at().subId() )
 		    { 
-			//Check type module version
+			//-- Check type module version --
 			if( AtMod.t_ver != owner().at(list[i_sub]).at().subVer() )
 			{
-			    Mess->put(nodePath().c_str(),TMess::Warning,Mess->I18N("%s for type <%s> no support module version: %d!"),
+			    mess_warning(nodePath().c_str(),_("%s for type <%s> no support module version: %d!"),
 				AtMod.id.c_str(),AtMod.type.c_str(),AtMod.t_ver);
 			    break;
 			}
-			//Check module present
+			//-- Check module present --
 			if( owner().at(list[i_sub]).at().modPresent(AtMod.id) )
-			    Mess->put(nodePath().c_str(),TMess::Warning,Mess->I18N("Module <%s> already present!"),AtMod.id.c_str());
+			    mess_warning(nodePath().c_str(),_("Module <%s> already present!"),AtMod.id.c_str());
 			else
 			{
-			    //Attach new module
+			    //-- Attach new module --
 			    TModule *LdMod = (attach)( AtMod, iname );
 			    if( LdMod == NULL )
 			    {
-				Mess->put(nodePath().c_str(),TMess::Warning,Mess->I18N("Attach module <%s> error!"),AtMod.id.c_str());
+				mess_warning(nodePath().c_str(),_("Attach module <%s> error!"),AtMod.id.c_str());
 				break;
 			    }
-			    //Add atached module
+			    //-- Add atached module --
 			    owner().at(list[i_sub]).at().modAdd(LdMod);
-			    SUse t_suse = { list[i_sub], LdMod->modId() };
-			    SchHD[i_sh]->use.push_back( t_suse );
+			    SchHD[i_sh]->use.push_back( list[i_sub]+"."+LdMod->modId() );
 			    if(full)
 			    {
 				owner().at(list[i_sub]).at().subLoad();
@@ -365,7 +369,7 @@ void TModSchedul::libAtt( const string &iname, bool full )
 	    else SchHD[i_sh]->hd = h_lib;
 	    return;
 	}
-    throw TError(nodePath().c_str(),Mess->I18N("SO <%s> no present!"),iname.c_str());
+    throw TError(nodePath().c_str(),_("SO <%s> no present!"),iname.c_str());
 }
 
 void TModSchedul::libDet( const string &iname )
@@ -379,12 +383,15 @@ void TModSchedul::libDet( const string &iname )
 	    {
 		try
 		{
-		    owner().at(SchHD[i_sh]->use[0].mod_sub).at().modAt(SchHD[i_sh]->use[0].n_mod).at().modStop();
-		    owner().at(SchHD[i_sh]->use[0].mod_sub).at().modDel( SchHD[i_sh]->use[0].n_mod );
+		    owner().at(TSYS::strSepParse(SchHD[i_sh]->use[0],0,'.')).at().
+			    modAt(TSYS::strSepParse(SchHD[i_sh]->use[0],1,'.')).at().modStop();
+		    owner().at(TSYS::strSepParse(SchHD[i_sh]->use[0],0,'.')).at().
+			    modDel(TSYS::strSepParse(SchHD[i_sh]->use[0],1,'.'));
 		}catch(TError err)
 		{   
 		    //owner().at(SchHD[i_sh]->use[0].mod_sub).at().modAt(SchHD[i_sh]->use[0].n_mod).at().modLoad();
-		    owner().at(SchHD[i_sh]->use[0].mod_sub).at().modAt(SchHD[i_sh]->use[0].n_mod).at().modStart();
+		    owner().at(TSYS::strSepParse(SchHD[i_sh]->use[0],0,'.')).at().
+			    modAt(TSYS::strSepParse(SchHD[i_sh]->use[0],1,'.')).at().modStart();
 		    throw;
 		}		
 		SchHD[i_sh]->use.erase(SchHD[i_sh]->use.begin());
@@ -394,10 +401,10 @@ void TModSchedul::libDet( const string &iname )
 	    return;
 	}
     }
-    throw TError(nodePath().c_str(),Mess->I18N("SO <%s> no present!"),iname.c_str());
+    throw TError(nodePath().c_str(),_("SO <%s> no present!"),iname.c_str());
 }
 
-bool TModSchedul::CheckAuto( const string &name) const
+bool TModSchedul::CheckAuto( const string &name ) const
 {
     if( m_am_list.size() == 1 && m_am_list[0] == "*") return(true);
     else 
@@ -422,7 +429,7 @@ TModSchedul::SHD &TModSchedul::lib( const string &iname )
     for(unsigned i_sh = 0; i_sh < SchHD.size(); i_sh++)
        	if( SchHD[i_sh]->name == iname ) 
 	    return *SchHD[i_sh];
-    throw TError(nodePath().c_str(),Mess->I18N("SO <%s> no present!"),iname.c_str());
+    throw TError(nodePath().c_str(),_("SO <%s> no present!"),iname.c_str());
 }
 
 void TModSchedul::libLoad( const string &iname, bool full)
@@ -441,8 +448,8 @@ void TModSchedul::libLoad( const string &iname, bool full)
 	    try { if(st_auto) libDet(files[i_f]); }
 	    catch(TError err) 
 	    { 
-		Mess->put(err.cat.c_str(),TMess::Warning,"%s",err.mess.c_str());
-		Mess->put(nodePath().c_str(),TMess::Warning,Mess->I18N("Can't detach library <%s>."),files[i_f].c_str());
+		mess_warning(err.cat.c_str(),"%s",err.mess.c_str());
+		mess_warning(nodePath().c_str(),_("Can't detach library <%s>."),files[i_f].c_str());
 		continue;
 	    }
 	}
@@ -450,59 +457,55 @@ void TModSchedul::libLoad( const string &iname, bool full)
 	if(st_auto) 
 	{
 	    try{ libAtt(files[i_f],full); }
-	    catch( TError err ){ Mess->put(err.cat.c_str(),TMess::Warning,"%s",err.mess.c_str()); }
+	    catch( TError err ){ mess_warning(err.cat.c_str(),"%s",err.mess.c_str()); }
 	}
     }
 }
 
-//================== Controll functions ========================
-void TModSchedul::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
+void TModSchedul::cntrCmdProc( XMLNode *opt )
 {
-    switch(cmd)
+    //- Get page info -
+    if( opt->name() == "info" )
     {
-	case TCntrNode::Info:
-	    TSubSYS::cntrCmd_( a_path, opt, cmd );       //Call parent
-
-	    ctrMkNode("area",opt,0,a_path.c_str(),"/ms",Mess->I18N("Subsystem"),0440);
-	    ctrMkNode("fld",opt,-1,a_path.c_str(),"/ms/chk_per",Mess->I18N("Check modules period (sec)"),0664,0,0,1,"tp","dec");
-	    ctrMkNode("comm",opt,-1,a_path.c_str(),"/ms/chk_now",Mess->I18N("Check modules now."));
-	    ctrMkNode("fld",opt,-1,a_path.c_str(),"/ms/mod_path",Mess->I18N("Path to shared libs(modules)"),0664,0,0,1,"tp","str");
-	    ctrMkNode("list",opt,-1,a_path.c_str(),"/ms/mod_auto",Mess->I18N("List of auto conected shared libs(modules)"),0664,0,0,2,"tp","str","s_com","add,ins,edit,del");
-	    ctrMkNode("comm",opt,-1,a_path.c_str(),"/ms/load",Mess->I18N("Load"));
-            ctrMkNode("comm",opt,-1,a_path.c_str(),"/ms/save",Mess->I18N("Save"));
-	    ctrMkNode("fld",opt,-1,a_path.c_str(),"/help/g_help",Mess->I18N("Options help"),0440,0,0,3,"tp","str","cols","90","rows","5");
-	    break;
-	case TCntrNode::Get:
-	    if( a_path == "/ms/chk_per" )	ctrSetI( opt, m_per );
-	    else if( a_path == "/ms/mod_path" )	ctrSetS( opt, m_mod_path );
-	    else if( a_path == "/ms/mod_auto" )
-	    {
-		opt->childClean();
-		for( unsigned i_a=0; i_a < m_am_list.size(); i_a++ )
-	    	    ctrSetS( opt, m_am_list[i_a] );
-	    }
-	    else if( a_path == "/help/g_help" )	ctrSetS( opt, optDescr() );       
-	    else TSubSYS::cntrCmd_( a_path, opt, cmd );
-	    break;
-	case TCntrNode::Set:
-	    if( a_path == "/ms/chk_per" )	chkPer(ctrGetI( opt ));
-	    else if( a_path == "/ms/chk_now" )	libLoad(m_mod_path,true);
-    	    else if( a_path == "/ms/mod_path" )	m_mod_path = ctrGetS( opt );
-	    else if( a_path == "/ms/load" )	subLoad( );
-	    else if( a_path == "/ms/save" )	subSave( );
-	    else if( a_path == "/ms/mod_auto" )
-	    {
-		if( opt->name() == "add" )		
-		    m_am_list.push_back(opt->text());
-		else if( opt->name() == "ins" )	
-		    m_am_list.insert(m_am_list.begin()+atoi(opt->attr("pos").c_str()),opt->text());
-		else if( opt->name() == "edit" )
-		    m_am_list[atoi(opt->attr("pos").c_str())] = opt->text();
-		else if( opt->name() == "del" )
-		    m_am_list.erase(m_am_list.begin()+atoi(opt->attr("pos").c_str()));
-	    }
-	    else TSubSYS::cntrCmd_( a_path, opt, cmd );
-	    break;
+	TSubSYS::cntrCmdProc(opt);
+        if(ctrMkNode("area",opt,0,"/ms",_("Subsystem"),0444,"root","root"))
+	{
+	    ctrMkNode("fld",opt,-1,"/ms/chk_per",_("Check modules period (sec)"),0664,"root","root",1,"tp","dec");
+	    ctrMkNode("comm",opt,-1,"/ms/chk_now",_("Check modules now."),0660,"root","root");
+	    ctrMkNode("fld",opt,-1,"/ms/mod_path",_("Path to shared libs(modules)"),0664,"root","root",1,"tp","str");
+	    ctrMkNode("list",opt,-1,"/ms/mod_auto",_("List of auto conected shared libs(modules)"),0664,"root","root",2,"tp","str","s_com","add,ins,edit,del");
+	    ctrMkNode("comm",opt,-1,"/ms/load",_("Load"),0660,"root","root");
+    	    ctrMkNode("comm",opt,-1,"/ms/save",_("Save"),0660,"root","root");
+	}
+	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","10");
+        return;
     }
+    
+    //- Process command to page -
+    string a_path = opt->attr("path");
+    if( a_path == "/ms/chk_per" )
+    {
+	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(TSYS::int2str(m_per));
+	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	chkPer(atoi(opt->text().c_str()));
+    }
+    else if( a_path == "/ms/mod_path" )	
+    {
+	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(m_mod_path);
+	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	m_mod_path = opt->text();
+    }
+    else if( a_path == "/ms/mod_auto" )
+    {
+	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )
+	    for( unsigned i_a=0; i_a < m_am_list.size(); i_a++ )
+    		opt->childAdd("el")->setText(m_am_list[i_a]);
+	if( ctrChkNode(opt,"add",0664,"root","root",SEQ_WR) )	m_am_list.push_back(opt->text());
+	if( ctrChkNode(opt,"ins",0664,"root","root",SEQ_WR) )	m_am_list.insert(m_am_list.begin()+atoi(opt->attr("pos").c_str()),opt->text());
+	if( ctrChkNode(opt,"edit",0664,"root","root",SEQ_WR) )	m_am_list[atoi(opt->attr("pos").c_str())] = opt->text();
+	if( ctrChkNode(opt,"del",0664,"root","root",SEQ_WR) )	m_am_list.erase(m_am_list.begin()+atoi(opt->attr("pos").c_str()));
+    }		
+    else if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440,"root","root",SEQ_RD) )	opt->setText(optDescr());
+    else if( a_path == "/ms/chk_now" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )	libLoad(m_mod_path,true);
+    else if( a_path == "/ms/load" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )		subLoad( );
+    else if( a_path == "/ms/save" && ctrChkNode(opt,"set",0660,"root","root",SEQ_WR) )		subSave( );           
+    else TSubSYS::cntrCmdProc(opt);
 }
-
