@@ -1,13 +1,12 @@
 
 //OpenSCADA system file: tmodule.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -34,6 +33,9 @@
 #include "tsubsys.h"
 #include "tmodule.h"
 
+//*************************************************
+//* TModule                                       *
+//*************************************************
 const char *TModule::l_info[] = 
     {"Modul","Name","Type","Source","Version","Author","Descript","License"};
 
@@ -44,7 +46,7 @@ TModule::TModule( )
 
 TModule::~TModule(  )
 {
-    //Clean export function list
+    //- Clean export function list -
     for(int i=0; i < m_efunc.size(); i++)
         delete m_efunc[i];
 }
@@ -54,11 +56,11 @@ string TModule::modName()
     return I18Ns(mName); 
 }
 
-void TModule::postEnable()
+void TModule::postEnable( int flag )
 {
-#if OSC_DEBUG 
-    Mess->put(nodePath().c_str(),TMess::Info,Mess->I18N("Connect module!"));
-#endif    
+    if( flag&TCntrNode::NodeRestore )	return;
+    
+    mess_info(nodePath().c_str(),_("Connect module!"));
 
     lc_id = string("oscd_")+mId;
     bindtextdomain(lc_id.c_str(),LOCALEDIR);
@@ -83,7 +85,7 @@ TModule::ExpFunc &TModule::modFunc( const string &prot )
 {
     for(int i=0; i < m_efunc.size(); i++)
 	if( m_efunc[i]->prot == prot ) return *m_efunc[i];
-    throw TError(nodePath().c_str(),Mess->I18N("Function <%s> no present into module!"),prot.c_str());        
+    throw TError(nodePath().c_str(),_("Function <%s> no present into module!"),prot.c_str());        
 }	
 
 void TModule::modFunc( const string &prot, void (TModule::**offptr)() )
@@ -110,51 +112,45 @@ string TModule::modInfo( const string &name )
     else if( name == l_info[6] ) info=I18Ns(mDescr);
     else if( name == l_info[7] ) info=I18Ns(mLicense);
     
-    return(info);
+    return info;
 }
 
-//==============================================================
-//================== Controll functions ========================
-//==============================================================
-void TModule::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
+void TModule::cntrCmdProc( XMLNode *opt )
 {
-    vector<string> list;
-    
-    if( cmd==TCntrNode::Info )
+    //- Get page info -
+    if( opt->name() == "info" )
     {
-	TCntrNode::cntrCmd_(a_path,opt,cmd);
-    	
-	ctrMkNode("oscada_cntr",opt,-1,a_path.c_str(),"/",Mess->I18N("Module: ")+modId());
-	if(TUIS::presentIco(owner().subId()+"."+modId()))
-	    ctrMkNode("img",opt,-1,a_path.c_str(),"/ico","",0444);
-	ctrMkNode("area",opt,-1,a_path.c_str(),"/help",Mess->I18N("Help"));
-	ctrMkNode("area",opt,-1,a_path.c_str(),"/help/m_inf",Mess->I18N("Module information"));
-    	
-	modInfo(list);
-	for( int i_l = 0; i_l < list.size(); i_l++)
-	    ctrMkNode("fld",opt,-1,a_path.c_str(),(string("/help/m_inf/")+list[i_l]).c_str(),I18Ns(list[i_l]),0444,0,0,1,"tp","str");	
-    }
-    else if( cmd==TCntrNode::Get )
+	ctrMkNode("oscada_cntr",opt,-1,"/",_("Module: ")+modId());
+	ctrMkNode("branches",opt,-1,"/br","",0444);
+	if(TUIS::icoPresent(owner().subId()+"."+modId())) ctrMkNode("img",opt,-1,"/ico","",0444);
+	if(ctrMkNode("area",opt,-1,"/help",_("Help")))
+	    if(ctrMkNode("area",opt,-1,"/help/m_inf",_("Module information")))
+	    {
+    		vector<string> list;
+		modInfo(list);
+		for( int i_l = 0; i_l < list.size(); i_l++)
+		    ctrMkNode("fld",opt,-1,(string("/help/m_inf/")+list[i_l]).c_str(),I18Ns(list[i_l]),0444,"root","root",1,"tp","str");
+	    }
+        return;    
+    } 
+       
+    //- Process command to page -
+    string a_path = opt->attr("path");
+    if( a_path == "/ico" && ctrChkNode(opt) )
     {
-	if(a_path == "/ico")
-	{
-	    string itp;
-	    opt->text(TSYS::strCode(TUIS::getIco(owner().subId()+"."+modId(),&itp),TSYS::base64));
-	    opt->attr("tp",itp);
-	}
-	else if( a_path.substr(0,11) == "/help/m_inf" )	ctrSetS( opt, modInfo(TSYS::pathLev(a_path,2)) ); 
-	else TCntrNode::cntrCmd_(a_path,opt,cmd);
+	string itp;
+	opt->setText(TSYS::strEncode(TUIS::icoGet(owner().subId()+"."+modId(),&itp),TSYS::base64));
+	opt->setAttr("tp",itp);
     }
-    else if( cmd==TCntrNode::Set )
-	TCntrNode::cntrCmd_(a_path,opt,cmd);
+    else if( a_path.substr(0,11) == "/help/m_inf" && ctrChkNode(opt) )	
+	opt->setText(modInfo(TSYS::pathLev(a_path,2)));
 }
 
-//================== Translate functions ======================
 const char *TModule::I18N( const char *mess )   
 { 
     const char *rez = Mess->I18N(mess,lc_id.c_str());
-    if( !strcmp(mess,rez) ) rez = Mess->I18N(mess);
-    return( rez ); 
+    if( !strcmp(mess,rez) ) rez = _(mess);
+    return rez; 
 }
 
 string TModule::I18Ns( const string &mess ) 
