@@ -1,13 +1,12 @@
 
 //OpenSCADA system file: tmodule.h
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -32,7 +31,9 @@
 using std::string;
 using std::vector;
 
-//Export functions object
+//*************************************************
+//* TModule                                       *
+//*************************************************
 class TSubSYS;
 
 class TModule : public TCntrNode 
@@ -40,14 +41,30 @@ class TModule : public TCntrNode
     friend class TSubSYS;
     public:
 	//Data
-	//- Attach module struct -
-	struct SAt
+	//*****************************************
+	//* SAt: Attach module struct             *
+	class SAt
 	{
-	    string id;          //id module
-	    string type;        //Type module
-	    int    t_ver;       //Type version module
+	    public:
+		//Methods
+		SAt( )	{ }
+		SAt( const string &iid, const string &itype = "", int itver = 0 ) :
+		    id(iid), type(itype), t_ver(itver)	{ }
+		    
+		bool operator==( const TModule::SAt &amst ) const
+		{
+		    if( amst.id == id && amst.type == type && amst.t_ver == t_ver ) return true;
+		    return false;
+		}
+		
+		//Attributes
+		string id;          //id module
+		string type;        //Type module
+		int    t_ver;       //Type version module
 	};
 	
+	//*****************************************
+	//* ExpFunc                               *	
 	class ExpFunc
 	{
 	    public:
@@ -65,10 +82,10 @@ class TModule : public TCntrNode
         const string &modId()   { return mId; }
 	string modName();		       
     
-	virtual void modLoad( ) { }
-	virtual void modSave( ) { }
-        virtual void modStart( ) { }
-        virtual void modStop( ) { }
+	virtual void modLoad( ) 	{ }
+	virtual void modSave( ) 	{ }
+        virtual void modStart( ) 	{ }
+        virtual void modStop( ) 	{ }
     
 	virtual void   modInfo( vector<string> &list );
 	virtual string modInfo( const string &name );
@@ -87,8 +104,8 @@ class TModule : public TCntrNode
     
     protected:    
 	//Methods
-	void postEnable();
-	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
+	void postEnable(int flag);
+	void cntrCmdProc( XMLNode *opt );       //Control interface command process
 	
 	//- Reg export function -
 	void modFuncReg( ExpFunc *func )	{ m_efunc.push_back(func); }
