@@ -1,13 +1,12 @@
 
 //OpenSCADA system file: tprotocols.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -26,9 +25,9 @@
 #include "tmess.h"
 #include "tprotocols.h"
 
-//================================================================
-//=========== TProtocolS =========================================
-//================================================================
+//************************************************
+//* TProtocolS                                   *
+//************************************************
 TProtocolS::TProtocolS( ) : TSubSYS("Protocol","Transport protocols",true) 
 {
 
@@ -41,13 +40,13 @@ TProtocolS::~TProtocolS( )
 
 string TProtocolS::optDescr(  )
 {
-    return(Mess->I18N(
+    return(_(
     	"=============== Subsystem \"Transport protocols\" options =================\n\n"));
 }
 
 void TProtocolS::subLoad()
 {
-    //========== Load parameters from command line ============
+    //- Load parameters from command line -
     int next_opt;
     char *short_opt="h";
     struct option long_opt[] =
@@ -67,34 +66,31 @@ void TProtocolS::subLoad()
 	}
     } while(next_opt != -1);
     
-    //========== Load parameters from config file =============
+    //- Load parameters from config file -
 
-    //Load modules
+    //- Load modules -
     TSubSYS::subLoad();
 }
 
-//=========== Control ==========================================
-void TProtocolS::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
+void TProtocolS::cntrCmdProc( XMLNode *opt )
 {
-    if( cmd==TCntrNode::Info )
+    //- Get page info -
+    if( opt->name() == "info" )
     {
-	TSubSYS::cntrCmd_( a_path, opt, cmd );       //Call parent
-
-	ctrMkNode("fld",opt,-1,a_path.c_str(),"/help/g_help",Mess->I18N("Options help"),0440,0,0,3,"tp","str","cols","90","rows","5");
+        TSubSYS::cntrCmdProc(opt);
+	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root",subId().c_str(),3,"tp","str","cols","90","rows","10");
+	return;
     }
-    else if( cmd==TCntrNode::Get )
-    {
-	if( a_path == "/help/g_help" ) ctrSetS( opt, optDescr() );
-	else TSubSYS::cntrCmd_( a_path, opt, cmd );
-    }
-    else if( cmd==TCntrNode::Set )
-    	TSubSYS::cntrCmd_( a_path, opt, cmd );
+    //- Process command to page -
+    string a_path = opt->attr("path");
+    if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440,"root",subId().c_str()) )	opt->setText(optDescr());
+    else TSubSYS::cntrCmdProc(opt);
 }
 
 
-//================================================================
-//=========== TProtocol ==========================================
-//================================================================
+//************************************************
+//* TProtocol                                    *
+//************************************************
 TProtocol::TProtocol()
 {
     m_pr = grpAdd("pr_");
@@ -115,9 +111,9 @@ void TProtocol::close( const string &name )
     chldDel(m_pr,name);
 }
 
-//================================================================
-//=========== TProtocolIn ========================================
-//================================================================
+//************************************************
+//* TProtocolIn                                  *
+//************************************************
 TProtocolIn::TProtocolIn( const string &name ) : m_name(name)
 {
 
