@@ -1,13 +1,12 @@
 
 //OpenSCADA system file: tsubsys.h
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -31,21 +30,23 @@
 using std::string;
 using std::vector;
 
+//*************************************************
+//* TSubSYS                                       *
+//*************************************************
 class TModSchedul;
 class TSYS;
 
 class TSubSYS : public TCntrNode
 {
     public:
-	//Methods
-	TSubSYS( char *id, char *name, bool modi = false );
+	//Public methods
+	TSubSYS( char *id, char *name, bool mod = false );
 	virtual ~TSubSYS(  );
 	
 	string subId()		{ return m_id; }
 	string subName();
 	
 	bool subModule()	{ return m_mod_sys; }	//Module subsystem
-	int subSecGrp();    	// Security group id
 	
 	virtual int subVer( )	{ return 0; }		//Type/grp module version
 	
@@ -56,24 +57,23 @@ class TSubSYS : public TCntrNode
 	virtual void subStop( );
     
 	//- Modules -
-	void modList( vector<string> &list )	{ chldList(m_mod,list); }
-        bool modPresent( const string &name )	{ return chldPresent(m_mod,name); }
+	void modList( vector<string> &list );
+        bool modPresent( const string &name );
 	void modAdd( TModule *modul );
 	void modDel( const string &name );
-        AutoHD<TModule> modAt( const string &name )
-	{ return chldAt(m_mod,name); }           
+        AutoHD<TModule> modAt( const string &name );
 	
 	TSYS &owner()		{ return *(TSYS *)nodePrev(); }
 
     protected:
-	//Methods
-	void cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd );
+	//Protected methods
+	void cntrCmdProc( XMLNode *opt );       //Control interface command process
 
     private:
-	//Methods
+	//Private methods
 	string nodeName()       { return subId(); }
 	
-	//Attributes
+	//Private attributes
 	bool	m_mod_sys;
 	int	m_mod;
 	
