@@ -1,13 +1,11 @@
-
 //OpenSCADA system file: tuis.cpp
 /***************************************************************************
- *   Copyright (C) 2003-2006 by Roman Savochenko                           *
+ *   Copyright (C) 2003-2007 by Roman Savochenko                           *
  *   rom_as@fromru.com                                                     *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   the Free Software Foundation; version 2 of the License.               *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
@@ -29,9 +27,9 @@
 #include "tmess.h"
 #include "tuis.h"
 
-//================================================================
-//================== TUIS ========================================
-//================================================================
+//*************************************************
+//* TUIS                                          *
+//*************************************************
 TUIS::TUIS( ) : TSubSYS("UI","User interfaces",true)
 {
 
@@ -39,13 +37,13 @@ TUIS::TUIS( ) : TSubSYS("UI","User interfaces",true)
 
 string TUIS::optDescr( )
 {
-    return(Mess->I18N(
+    return(_(
     	"===================== Subsystem \"User interfaces\" options ===============\n\n"));
 }
 
 void TUIS::subLoad( )
 {
-    //========== Load parameters from command line ============
+    //- Load parameters from command line -
     int next_opt;
     char *short_opt="h";
     struct option long_opt[] =
@@ -65,15 +63,33 @@ void TUIS::subLoad( )
 	}
     } while(next_opt != -1);
     
-    //========== Load parameters from config file =============
+    //- Load parameters from config file -
 
-    //Load modules
+    //- Load modules -
     TSubSYS::subLoad();
 }
 
-bool TUIS::presentIco(const string &inm, string *tp)
+void TUIS::subStart(  )
 {
-    int hd = open((string("./icons/")+inm+".png").c_str(),O_RDONLY);
+#if OSC_DEBUG
+    mess_debug(nodePath().c_str(),_("Start subsystem."));
+#endif
+
+    TSubSYS::subStart( );
+}
+    
+void TUIS::subStop( )
+{
+#if OSC_DEBUG
+    mess_debug(nodePath().c_str(),_("Stop subsystem."));
+#endif
+
+    TSubSYS::subStop( );
+}
+
+bool TUIS::icoPresent(const string &inm, string *tp)
+{
+    int hd = open(icoPath(inm).c_str(),O_RDONLY);
     if( hd != -1 )
     {
 	if( tp ) *tp = "png";
@@ -83,13 +99,13 @@ bool TUIS::presentIco(const string &inm, string *tp)
     return false;    
 }
 
-string TUIS::getIco(const string &inm, string *tp )
+string TUIS::icoGet(const string &inm, string *tp )
 {
     int len;
     char buf[STR_BUF_LEN];
     string rez;
     
-    int hd = open((string("./icons/")+inm+".png").c_str(),O_RDONLY);
+    int hd = open(icoPath(inm).c_str(),O_RDONLY);
     if( hd != -1 )
     {
         if( tp ) *tp = "png";
@@ -101,51 +117,52 @@ string TUIS::getIco(const string &inm, string *tp )
     return rez;
 }
 
-void TUIS::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
+string TUIS::icoPath(const string &ico)
 {
-    if( cmd==TCntrNode::Info )
-    {
-	TSubSYS::cntrCmd_( a_path, opt, cmd );       //Call parent
-
-	ctrMkNode("fld",opt,-1,a_path.c_str(),"/help/g_help",Mess->I18N("Options help"),0440,0,0,3,"tp","str","cols","90","rows","5");
-    }
-    else if( cmd==TCntrNode::Get )
-    {
-    	if( a_path == "/help/g_help" ) ctrSetS( opt, optDescr() );
-	else TSubSYS::cntrCmd_( a_path, opt, cmd );
-    }
-    else if( cmd==TCntrNode::Set )
-	TSubSYS::cntrCmd_( a_path, opt, cmd );
+    return "./icons/"+ico+".png";
 }
 
+void TUIS::cntrCmdProc( XMLNode *opt )
+{
+    //- Get page info -
+    if( opt->name() == "info" )
+    {
+        TSubSYS::cntrCmdProc(opt);
+	ctrMkNode("fld",opt,-1,"/help/g_help",_("Options help"),0440,"root","root",3,"tp","str","cols","90","rows","10");
+	return;
+    }
+    //- Process command to page -
+    string a_path = opt->attr("path");
+    if( a_path == "/help/g_help" && ctrChkNode(opt,"get",0440) ) opt->setText(optDescr());
+    else TSubSYS::cntrCmdProc(opt);
+}		    
 
-//================================================================
-//================== TUI =========================================
-//================================================================
+//*************************************************
+//* TUI                                           *
+//*************************************************
 TUI::TUI() : run_st(false)
 {
 
 }
-    
-void TUI::cntrCmd_( const string &a_path, XMLNode *opt, TCntrNode::Command cmd )
-{
-    if( cmd==TCntrNode::Info )
-    {
-	TModule::cntrCmd_( a_path, opt, cmd );       //Call parent
-	
-	ctrMkNode("area",opt,0,a_path.c_str(),"/prm",Mess->I18N("User interface"));
-	ctrMkNode("area",opt,-1,a_path.c_str(),"/prm/st",Mess->I18N("State"));
-	ctrMkNode("fld",opt,-1,a_path.c_str(),"/prm/st/r_st",Mess->I18N("Runing"),0664,0,0,1,"tp","bool");
-    }
-    else if( cmd==TCntrNode::Get )
-    {
-	if( a_path == "/prm/st/r_st" )	ctrSetB( opt, run_st );
-	else TModule::cntrCmd_( a_path, opt, cmd );
-    }
-    else if( cmd==TCntrNode::Set )
-    {
-	if( a_path == "/prm/st/r_st" )	ctrGetB( opt )?modStart():modStop();
-	else TModule::cntrCmd_( a_path, opt, cmd );
-    }
-}
 
+void TUI::cntrCmdProc( XMLNode *opt )
+{
+    //- Get page info -
+    if( opt->name() == "info" )
+    {
+        TModule::cntrCmdProc(opt);
+	if(ctrMkNode("area",opt,0,"/prm",_("User interface")))
+	    if(ctrMkNode("area",opt,-1,"/prm/st",_("State")))
+		ctrMkNode("fld",opt,-1,"/prm/st/r_st",_("Runing"),0664,"root","root",1,"tp","bool");
+	return;
+    }
+    
+    //- Process command to page -
+    string a_path = opt->attr("path");
+    if( a_path == "/prm/st/r_st" )
+    {
+	if( ctrChkNode(opt,"get",0664,"root","root",SEQ_RD) )	opt->setText(run_st?"1":"0");
+	if( ctrChkNode(opt,"set",0664,"root","root",SEQ_WR) )	atoi(opt->text().c_str())?modStart():modStop();
+    }
+    else TModule::cntrCmdProc(opt);
+}		    
